@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 1999-2011 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,16 +22,7 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkwidget.h>
-#include <gtk/gtktreestore.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkhbbox.h>
-#include <gtk/gtksignal.h>
+#include <gtk/gtk.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
@@ -71,9 +62,9 @@ struct _FolderItemSearch
 	GtkTreeIter iter;
 };
 
-static GdkPixbuf *folder_pixbuf;
-static GdkPixbuf *folderopen_pixbuf;
-static GdkPixbuf *foldernoselect_pixbuf;
+static GdkPixbuf *folder_pixbuf = NULL;
+static GdkPixbuf *folderopen_pixbuf = NULL;
+static GdkPixbuf *foldernoselect_pixbuf = NULL;
 
 static GtkWidget *window;
 static GtkWidget *treeview;
@@ -188,6 +179,7 @@ FolderItem *foldersel_folder_sel(Folder *cur_folder, FolderSelectionType type,
 	gtk_widget_grab_focus(treeview);
 
 	gtk_widget_show(window);
+	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
 	manage_window_set_transient(GTK_WINDOW(window));
 
 	cancelled = finished = FALSE;
@@ -196,6 +188,7 @@ FolderItem *foldersel_folder_sel(Folder *cur_folder, FolderSelectionType type,
 		gtk_main_iteration();
 
 	gtk_widget_hide(window);
+	gtk_window_set_modal(GTK_WINDOW(window), FALSE);
 	gtk_entry_set_text(GTK_ENTRY(entry), "");
 	gtk_tree_store_clear(tree_store);
 
@@ -210,7 +203,7 @@ FolderItem *foldersel_folder_sel(Folder *cur_folder, FolderSelectionType type,
 static void foldersel_size_allocate_cb(GtkWidget *widget,
 					 GtkAllocation *allocation)
 {
-	g_return_if_fail(allocation != NULL);
+	cm_return_if_fail(allocation != NULL);
 
 	prefs_common.folderselwin_width = allocation->width;
 	prefs_common.folderselwin_height = allocation->height;
@@ -230,7 +223,6 @@ static void foldersel_create(void)
 	gtk_window_set_title(GTK_WINDOW(window), _("Select folder"));
 	gtk_container_set_border_width(GTK_CONTAINER(window), 4);
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
-	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
 	gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
 
 	gtk_widget_realize(window);
@@ -269,6 +261,8 @@ static void foldersel_create(void)
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview), FALSE);
 	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview),
 	                             prefs_common.use_stripes_everywhere);
+	gtk_tree_view_set_enable_tree_lines(GTK_TREE_VIEW(treeview),
+							prefs_common.enable_dotted_lines);
 	gtk_tree_view_set_search_column(GTK_TREE_VIEW(treeview),
 					FOLDERSEL_FOLDERNAME);
 
@@ -307,7 +301,7 @@ static void foldersel_create(void)
 	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
 
 	entry = gtk_entry_new();
-	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+	gtk_editable_set_editable(GTK_EDITABLE(entry), FALSE);
 	gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(entry), "activate",
 			 G_CALLBACK(foldersel_entry_activated), NULL);
@@ -350,6 +344,17 @@ static void foldersel_init(void)
 			 &foldernoselect_pixbuf);
 }
 
+void foldersel_reflect_prefs_pixmap_theme(void)
+{
+	if (folder_pixbuf)
+		g_object_unref(folder_pixbuf);
+	if (folderopen_pixbuf)
+		g_object_unref(folderopen_pixbuf);
+	if (foldernoselect_pixbuf)
+		g_object_unref(foldernoselect_pixbuf);
+	foldersel_init();
+}
+
 static void foldersel_append_item(GtkTreeStore *store, FolderItem *item,
 				  GtkTreeIter *iter, GtkTreeIter *parent)
 {
@@ -443,9 +448,9 @@ static void foldersel_insert_gnode_in_store(GtkTreeStore *store, GNode *node,
 	GtkTreeIter child;
 	GNode *iter;
 
-	g_return_if_fail(node != NULL);
-	g_return_if_fail(node->data != NULL);
-	g_return_if_fail(store != NULL);
+	cm_return_if_fail(node != NULL);
+	cm_return_if_fail(node->data != NULL);
+	cm_return_if_fail(store != NULL);
 
 	item = FOLDER_ITEM(node->data);
 	foldersel_append_item(store, item, &child, parent);
@@ -462,7 +467,7 @@ static void foldersel_set_tree(Folder *cur_folder, FolderSelectionType type)
 
 	for (list = folder_get_list(); list != NULL; list = list->next) {
 		folder = FOLDER(list->data);
-		g_return_if_fail(folder != NULL);
+		cm_return_if_fail(folder != NULL);
 
 		if (type != FOLDER_SEL_ALL) {
 			if (FOLDER_TYPE(folder) == F_NEWS)
