@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto and the Claws Mail Team
+ * Copyright (C) 1999-2011 Hiroyuki Yamamoto and the Claws Mail Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 #include <glib/gi18n.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <gtk/gtktext.h>
 #include <gdk/gdkkeysyms.h>
 
 #include "utils.h"
@@ -51,7 +50,6 @@ struct _PrefsWindow
 	GtkWidget *vbox;
 	GtkWidget *paned;
 	GtkWidget *scrolledwindow1;
-	GtkWidget *scrolledwindow2;
 	GtkWidget *tree_view;
 	GtkWidget *table2;
 	GtkWidget *pagelabel;
@@ -71,7 +69,7 @@ struct _PrefsWindow
 
 	gpointer   	 data;
 	GSList	  	*prefs_pages;
-	GtkDestroyNotify func;
+	GDestroyNotify func;
 };
 
 struct _PrefsTreeNode
@@ -132,7 +130,7 @@ static void close_all_pages(GSList *prefs_pages)
 	}	
 }
 
-#ifdef MAEMO
+#ifdef GENERIC_UMPC
 static void prefs_show_sections(PrefsWindow *prefswindow)
 {
 	gint max;
@@ -167,7 +165,7 @@ static void apply_button_clicked(GtkButton *button, gpointer user_data)
 	PrefsWindow *prefswindow = (PrefsWindow *) user_data;
 
 	save_all_pages(prefswindow->prefs_pages);
-#ifdef MAEMO
+#ifdef GENERIC_UMPC
 	prefs_show_sections(prefswindow);
 #endif
 }
@@ -209,7 +207,7 @@ static gboolean window_closed(GtkWidget *widget, GdkEvent *event, gpointer user_
 {
 	PrefsWindow *prefswindow = (PrefsWindow *) user_data;
 
-#ifdef MAEMO
+#ifdef GENERIC_UMPC
 	save_all_pages(prefswindow->prefs_pages);
 #endif
 	close_prefs_window(prefswindow);
@@ -290,7 +288,30 @@ static gint prefswindow_tree_sort_by_weight(GtkTreeModel *model,
 	return f1 < f2 ? -1 : (f1 > f2 ?  1 : 
 	      (i1 < i2 ?  1 : (i1 > i2 ? -1 : 0)));
 }
-				  
+
+static void prefswindow_build_page(PrefsWindow *prefswindow, PrefsPage *page)
+{
+	GtkWidget *scrolledwin, *tmp;
+
+	if (!page->page_open) {
+		scrolledwin = gtk_scrolled_window_new(NULL, NULL);
+		gtk_widget_show(scrolledwin);
+		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwin),
+			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+		
+		page->create_widget(page, GTK_WINDOW(prefswindow->window), prefswindow->data);
+		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolledwin),
+					page->widget);
+
+		gtk_container_add(GTK_CONTAINER(prefswindow->notebook), scrolledwin);
+		tmp = gtk_bin_get_child(GTK_BIN(scrolledwin));
+		gtk_viewport_set_shadow_type(GTK_VIEWPORT(tmp), GTK_SHADOW_NONE);
+		
+		page->widget	= scrolledwin;
+		page->page_open	= TRUE;
+	}
+}
+
 static void prefswindow_build_all_pages(PrefsWindow *prefswindow, GSList *prefs_pages)
 {
 	GSList *cur;
@@ -299,25 +320,20 @@ static void prefswindow_build_all_pages(PrefsWindow *prefswindow, GSList *prefs_
 	for (cur = prefs_pages; cur != NULL; cur = g_slist_next(cur)) {
 		PrefsPage *page = (PrefsPage *) cur->data;
 
-		if (!page->page_open) {
-			page->create_widget(page, GTK_WINDOW(prefswindow->window), prefswindow->data);
-			gtk_container_add(GTK_CONTAINER(prefswindow->notebook), page->widget);
-			page->page_open = TRUE;
-		}
+		prefswindow_build_page(prefswindow, page);
 	}
 	prefs_pages = g_slist_reverse(prefs_pages);
 }
 
 static void prefswindow_build_tree(GtkWidget *tree_view, GSList *prefs_pages,
 									PrefsWindow *prefswindow,
-									gboolean preload_pages,
-									gboolean activate_child)
+									gboolean preload_pages)
 {
 	GtkTreeStore *store = GTK_TREE_STORE(gtk_tree_view_get_model
 			(GTK_TREE_VIEW(tree_view)));
 	GSList *cur;
 	gint index; /* index in pages list */
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	GtkTreeSelection *selection;
 	GtkTreeIter iter;
 #endif
@@ -339,7 +355,7 @@ static void prefswindow_build_tree(GtkWidget *tree_view, GSList *prefs_pages,
 			gtk_tree_model_foreach(GTK_TREE_MODEL(store), 
 					       (GtkTreeModelForeachFunc) find_node_by_name,
 					       &find_name);
-			if (find_name.found) {
+			if (find_name.found && page->path[i] != page->path[i-1]) {
 				node = find_name.node;
 				gtk_tree_model_get(GTK_TREE_MODEL(store), &node,
 						   PREFS_PAGE_DATA, &prefs_node,
@@ -399,10 +415,10 @@ static void prefswindow_build_tree(GtkWidget *tree_view, GSList *prefs_pages,
 		prefswindow_build_all_pages(prefswindow, prefs_pages);
 
 	/* select first one or its first child if necessary */
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
 	if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter)) {
-		if (activate_child && gtk_tree_model_iter_has_child(GTK_TREE_MODEL(store), &iter)) {
+		if (gtk_tree_model_iter_has_child(GTK_TREE_MODEL(store), &iter)) {
 			GtkTreeIter parent = iter;
 			if (!gtk_tree_model_iter_children(GTK_TREE_MODEL(store), &iter, &parent))
 				iter = parent;
@@ -413,9 +429,9 @@ static void prefswindow_build_tree(GtkWidget *tree_view, GSList *prefs_pages,
 }
 
 void prefswindow_open_full(const gchar *title, GSList *prefs_pages,
-							 gpointer data, GtkDestroyNotify func,
+							 gpointer data, GDestroyNotify func,
 							 gint *save_width, gint *save_height,
-							 gboolean preload_pages, gboolean activate_child,
+							 gboolean preload_pages,
 							 PrefsOpenCallbackFunc open_cb,
 							 PrefsCloseCallbackFunc close_cb)
 {
@@ -424,7 +440,6 @@ void prefswindow_open_full(const gchar *title, GSList *prefs_pages,
 	gint y = gdk_screen_height();
 	static GdkGeometry geometry;
 	GtkAdjustment *adj;
-	GtkWidget *tmp;
 
 	prefswindow = g_new0(PrefsWindow, 1);
 
@@ -489,21 +504,13 @@ void prefswindow_open_full(const gchar *title, GSList *prefs_pages,
 	gtk_misc_set_alignment(GTK_MISC(prefswindow->pagelabel), 0, 0.0);
 	gtk_container_add(GTK_CONTAINER(prefswindow->labelframe), prefswindow->pagelabel);
 
-	prefswindow->scrolledwindow2 = gtk_scrolled_window_new(NULL, NULL);
-	gtk_widget_show(prefswindow->scrolledwindow2);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(prefswindow->scrolledwindow2),
-			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-
 	prefswindow->notebook = gtk_notebook_new();
 	gtk_widget_show(prefswindow->notebook);
 	gtk_notebook_set_scrollable(GTK_NOTEBOOK(prefswindow->notebook), TRUE);
 	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(prefswindow->notebook), FALSE);
 	gtk_notebook_set_show_border(GTK_NOTEBOOK(prefswindow->notebook), FALSE);
-	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(prefswindow->scrolledwindow2),
-					prefswindow->notebook);
-	tmp = gtk_bin_get_child(GTK_BIN(prefswindow->scrolledwindow2));
-	gtk_viewport_set_shadow_type(GTK_VIEWPORT(tmp), GTK_SHADOW_NONE);
-	gtk_table_attach(GTK_TABLE(prefswindow->table2), prefswindow->scrolledwindow2,
+
+	gtk_table_attach(GTK_TABLE(prefswindow->table2), prefswindow->notebook,
 			0, 1, 1, 2, GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 0, 4);
 
 	prefswindow->empty_page = gtk_label_new("");
@@ -511,14 +518,14 @@ void prefswindow_open_full(const gchar *title, GSList *prefs_pages,
 	gtk_container_add(GTK_CONTAINER(prefswindow->notebook), prefswindow->empty_page);
 
 	prefswindow_build_tree(prefswindow->tree_view, prefs_pages, prefswindow,
-							preload_pages, activate_child);
+							preload_pages);
 
 	if (open_cb)
 		open_cb(GTK_WINDOW(prefswindow->window));
 
 	gtk_widget_grab_focus(prefswindow->tree_view);
 
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	gtkut_stock_button_set_create(&prefswindow->confirm_area,
 				      &prefswindow->apply_btn,	GTK_STOCK_APPLY,
 				      &prefswindow->cancel_btn,	GTK_STOCK_CANCEL,
@@ -532,11 +539,10 @@ void prefswindow_open_full(const gchar *title, GSList *prefs_pages,
 	gtk_widget_show_all(prefswindow->confirm_area);
 	gtk_widget_show(prefswindow->vbox);
 	gtk_widget_show(prefswindow->scrolledwindow1);
-	gtk_widget_show(prefswindow->scrolledwindow2);
 
 	gtk_box_pack_start(GTK_BOX(prefswindow->vbox), prefswindow->confirm_area, FALSE, FALSE, 0);
 
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	g_signal_connect(G_OBJECT(prefswindow->ok_btn), "clicked", 
 			 G_CALLBACK(ok_button_clicked), prefswindow);
 	g_signal_connect(G_OBJECT(prefswindow->cancel_btn), "clicked", 
@@ -583,7 +589,7 @@ void prefswindow_open_full(const gchar *title, GSList *prefs_pages,
 					    *(prefswindow->save_height));
 	}
 
-#ifdef MAEMO
+#ifdef GENERIC_UMPC
 	prefs_show_sections(prefswindow);
 #endif
 	gtk_widget_show(prefswindow->window);
@@ -594,14 +600,6 @@ void prefswindow_open_full(const gchar *title, GSList *prefs_pages,
 			GTK_SCROLLED_WINDOW(prefswindow->scrolledwindow1));
 	gtk_adjustment_set_value(adj, adj->lower);
 	gtk_adjustment_changed(adj);
-	adj = gtk_scrolled_window_get_vadjustment(
-			GTK_SCROLLED_WINDOW(prefswindow->scrolledwindow2));
-	gtk_adjustment_set_value(adj, adj->lower);
-	gtk_adjustment_changed(adj);
-	adj = gtk_scrolled_window_get_hadjustment(
-			GTK_SCROLLED_WINDOW(prefswindow->scrolledwindow2));
-	gtk_adjustment_set_value(adj, adj->lower);
-	gtk_adjustment_changed(adj);
 }
 
 void prefswindow_open(const gchar *title, GSList *prefs_pages, gpointer data,
@@ -610,7 +608,7 @@ void prefswindow_open(const gchar *title, GSList *prefs_pages, gpointer data,
 					 PrefsCloseCallbackFunc close_cb)
 {
 	prefswindow_open_full(title, prefs_pages, data, NULL, save_width, save_height,
-						  FALSE, FALSE, open_cb, close_cb);
+						  FALSE, open_cb, close_cb);
 }
 
 /*!
@@ -621,7 +619,7 @@ static void prefs_size_allocate_cb(GtkWidget *widget,
 {
 	PrefsWindow *prefswindow = (PrefsWindow *) user_data;
 
-	g_return_if_fail(allocation != NULL);
+	cm_return_if_fail(allocation != NULL);
 
 	/* don't try to save size to NULL pointers */
 	if (prefswindow && prefswindow->save_width && prefswindow->save_height) {
@@ -691,7 +689,9 @@ static gboolean prefswindow_row_selected(GtkTreeSelection *selector,
 	GtkTreeIter iter;
 	GtkAdjustment *adj;
 
-#ifndef MAEMO
+	if (gtk_tree_path_get_depth(path) < 2)
+		return FALSE;
+#ifndef GENERIC_UMPC
 	if (currently_selected) 
 		return TRUE;
 #endif
@@ -712,11 +712,7 @@ static gboolean prefswindow_row_selected(GtkTreeSelection *selector,
 		return TRUE;
 	}
 
-	if (!page->page_open) {
-		page->create_widget(page, GTK_WINDOW(prefswindow->window), prefswindow->data);
-		gtk_container_add(GTK_CONTAINER(prefswindow->notebook), page->widget);
-		page->page_open = TRUE;
-	}
+	prefswindow_build_page(prefswindow, page);
 
 	i = 0;
 	while (page->path[i + 1] != 0)
@@ -731,15 +727,15 @@ static gboolean prefswindow_row_selected(GtkTreeSelection *selector,
 				      pagenum);
 
 	adj = gtk_scrolled_window_get_vadjustment(
-			GTK_SCROLLED_WINDOW(prefswindow->scrolledwindow2));
+			GTK_SCROLLED_WINDOW(page->widget));
 	gtk_adjustment_set_value(adj, adj->lower);
 	gtk_adjustment_changed(adj);
 	adj = gtk_scrolled_window_get_hadjustment(
-			GTK_SCROLLED_WINDOW(prefswindow->scrolledwindow2));
+			GTK_SCROLLED_WINDOW(page->widget));
 	gtk_adjustment_set_value(adj, adj->lower);
 	gtk_adjustment_changed(adj);
 
-#ifdef MAEMO
+#ifdef GENERIC_UMPC
 	prefs_show_page(prefswindow);
 #endif
 	return TRUE;
