@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 1999-2011 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,8 +25,7 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkrc.h>
+#include <gtk/gtk.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -44,11 +43,6 @@
 #ifdef HAVE_LIBSM
 #include <X11/SM/SMlib.h>
 #endif
-#ifdef USE_GNUTLS
-#include <pthread.h>
-#include <gcrypt.h>
-GCRY_THREAD_OPTION_PTHREAD_IMPL;
-#endif
 
 #include "wizard.h"
 #ifdef HAVE_STARTUP_NOTIFICATION
@@ -57,10 +51,15 @@ GCRY_THREAD_OPTION_PTHREAD_IMPL;
 # include <gdk/gdkx.h>
 #endif
 
-#ifdef HAVE_NETWORKMANAGER_SUPPORT
+#ifdef HAVE_DBUS_GLIB
 #include <dbus/dbus-glib.h>
+#endif
+#ifdef HAVE_NETWORKMANAGER_SUPPORT
 #include <NetworkManager.h>
 #endif
+#ifdef HAVE_VALGRIND
+#include <valgrind.h>
+#endif
 
 #include "claws.h"
 #include "main.h"
@@ -109,15 +108,16 @@ GCRY_THREAD_OPTION_PTHREAD_IMPL;
 #include "news_gtk.h"
 #include "matcher.h"
 #include "tags.h"
+#include "hooks.h"
+#include "menu.h"
+#include "quicksearch.h"
+
 #ifdef HAVE_LIBETPAN
 #include "imap-thread.h"
 #include "nntp-thread.h"
 #endif
 #include "stock_pixmap.h"
-#ifdef HAVE_VALGRIND
-#include "valgrind.h"
-#endif
-#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
+#ifdef USE_GNUTLS
 #  include "ssl.h"
 #endif
 
@@ -135,7 +135,6 @@ GCRY_THREAD_OPTION_PTHREAD_IMPL;
 #include <hildon-widgets/hildon-banner.h>
 #include <hildon-widgets/hildon-program.h>
 #endif
-#include <gtk/gtkmain.h>
 #include <libosso.h>
 #include <libgnomevfs/gnome-vfs-volume.h>
 #include <libgnomevfs/gnome-vfs-volume-monitor.h>
@@ -161,6 +160,14 @@ static GnomeVFSVolumeMonitor *volmon;
 static gboolean went_offline_nm;
 #endif
 
+#if !defined(NM_CHECK_VERSION)
+#define NM_CHECK_VERSION(x,y,z) 0
+#endif
+
+#ifdef HAVE_DBUS_GLIB
+static DBusGProxy *awn_proxy = NULL;
+#endif
+
 gchar *prog_version;
 gchar *argv0;
 
@@ -189,6 +196,11 @@ static struct RemoteCmd {
 	gboolean compose;
 	const gchar *compose_mailto;
 	GPtrArray *attach_files;
+	gboolean search;
+	const gchar *search_folder;
+	const gchar *search_type;
+	const gchar *search_request;
+	gboolean search_recursive;
 	gboolean status;
 	gboolean status_full;
 	GPtrArray *status_folders;
@@ -314,12 +326,12 @@ static void startup_notification_complete(gboolean with_window)
 		 * if we have been launched from a menu.
 		 * We have to display a window, so let it be very little */
 		hack = gtk_window_new(GTK_WINDOW_POPUP);
-		gtk_widget_set_uposition(hack, 0, 0);
+		gtk_window_move(GTK_WINDOW(hack), 0, 0);
 		gtk_widget_set_size_request(hack, 1, 1);
 		gtk_widget_show(hack);
 	}
 
-	xdisplay = GDK_DISPLAY();
+	xdisplay = GDK_DISPLAY_XDISPLAY(gdk_display_get_default());
 	sn_display = sn_display_new(xdisplay,
 				sn_error_trap_push,
 				sn_error_trap_pop);
@@ -412,17 +424,16 @@ static gboolean migrate_old_config(const gchar *old_cfg_dir, const gchar *new_cf
 	gint r = 0;
 	GtkWidget *window = NULL;
 	GtkWidget *keep_backup_chk;
-	GtkTooltips *tips = gtk_tooltips_new();
+	CLAWS_TIP_DECL();
 	gboolean backup = TRUE;
 
 	keep_backup_chk = gtk_check_button_new_with_label (_("Keep old configuration"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(keep_backup_chk), TRUE);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tips), keep_backup_chk,
+	CLAWS_SET_TIP(keep_backup_chk,
 			     _("Keeping a backup will allow you to go back to an "
 			       "older version, but may take a while if you have "
 			       "cached IMAP or News data, and will take some extra "
-			       "room on your disk."),
-			     NULL);
+			       "room on your disk."));
 
 	g_signal_connect(G_OBJECT(keep_backup_chk), "toggled", 
 			G_CALLBACK(chk_update_val), &backup);
@@ -792,11 +803,16 @@ static void win32_log(const gchar *log_domain, GLogLevelFlags log_level, const g
 
 static void win32_open_log(void)
 {
-	if (is_file_exist("claws-win32.log")) {
-		if (rename_force("claws-win32.log", "claws-win32.log.bak") < 0)
-			FILE_OP_ERROR("claws-win32.log", "rename");
+	gchar *logfile = g_strconcat(g_get_tmp_dir(), G_DIR_SEPARATOR_S, "claws-win32.log", NULL);
+	gchar *oldlogfile = g_strconcat(g_get_tmp_dir(), G_DIR_SEPARATOR_S, "claws-win32.log.bak", NULL);
+
+	if (is_file_exist(logfile)) {
+		if (rename_force(logfile, oldlogfile) < 0)
+			FILE_OP_ERROR(logfile, "rename");
 	}
-	win32_debug_fp = fopen("claws-win32.log", "w");
+	win32_debug_fp = g_fopen(logfile, "w");
+	g_free(logfile);
+	g_free(oldlogfile);
 	if (win32_debug_fp)
 	{
 		g_set_print_handler(win32_print_stdout);
@@ -830,13 +846,22 @@ static void main_dump_features_list(gboolean show_debug_only)
 		return;
 
 	if (show_debug_only)
-		debug_print("GTK+ %d.%d.%d / GLib %d.%d.%d\n",
+		debug_print("runtime GTK+ %d.%d.%d / GLib %d.%d.%d\n",
 			   gtk_major_version, gtk_minor_version, gtk_micro_version,
 			   glib_major_version, glib_minor_version, glib_micro_version);
 	else
-		g_print("GTK+ %d.%d.%d / GLib %d.%d.%d\n",
+		g_print("runtime GTK+ %d.%d.%d / GLib %d.%d.%d\n",
 			   gtk_major_version, gtk_minor_version, gtk_micro_version,
 			   glib_major_version, glib_minor_version, glib_micro_version);
+	if (show_debug_only)
+		debug_print("buildtime GTK+ %d.%d.%d / GLib %d.%d.%d\n",
+			   GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION,
+			   GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
+	else
+		g_print("buildtime GTK+ %d.%d.%d / GLib %d.%d.%d\n",
+			   GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION,
+			   GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
+	
 	if (show_debug_only)
 		debug_print("Compiled-in features:\n");
 	else
@@ -847,7 +872,7 @@ static void main_dump_features_list(gboolean show_debug_only)
 	else
 		g_print(" compface\n");
 #endif
-#if USE_ASPELL
+#if USE_ENCHANT
 	if (show_debug_only)
 		debug_print(" aspell\n");
 	else
@@ -889,12 +914,6 @@ static void main_dump_features_list(gboolean show_debug_only)
 	else
 		g_print(" libetpan %d.%d\n", LIBETPAN_VERSION_MAJOR, LIBETPAN_VERSION_MINOR);
 #endif
-#if USE_GNOMEPRINT
-	if (show_debug_only)
-		debug_print(" gnomeprint\n");
-	else
-		g_print(" gnomeprint\n");
-#endif
 #if HAVE_LIBSM
 	if (show_debug_only)
 		debug_print(" libsm\n");
@@ -907,24 +926,120 @@ static void main_dump_features_list(gboolean show_debug_only)
 	else
 		g_print(" NetworkManager\n");
 #endif
-#if USE_OPENSSL
-	if (show_debug_only)
-		debug_print(" openssl\n");
-	else
-		g_print(" openssl\n");
-#endif
 }
 
+#ifdef HAVE_DBUS_GLIB
+static guint dbus_item_hook_id = -1;
+static guint dbus_folder_hook_id = -1;
+
+static void uninstall_dbus_status_handler(void)
+{
+	if(awn_proxy)
+		g_object_unref(awn_proxy);
+	awn_proxy = NULL;
+	if (dbus_item_hook_id != -1)
+		hooks_unregister_hook(FOLDER_ITEM_UPDATE_HOOKLIST, dbus_item_hook_id);
+	if (dbus_folder_hook_id != -1)
+		hooks_unregister_hook(FOLDER_UPDATE_HOOKLIST, dbus_folder_hook_id);
+}
+
+static void dbus_update(FolderItem *removed_item)
+{
+	guint new, unread, unreadmarked, marked, total;
+	guint replied, forwarded, locked, ignored, watched;
+	gchar *buf;
+	GError *error = NULL;
+
+	folder_count_total_msgs(&new, &unread, &unreadmarked, &marked, &total,
+				&replied, &forwarded, &locked, &ignored,
+				&watched);
+	if (removed_item) {
+		total -= removed_item->total_msgs;
+		new -= removed_item->new_msgs;
+		unread -= removed_item->unread_msgs;
+	}
+
+	if (new > 0) {
+		buf = g_strdup_printf("%d", new);
+		dbus_g_proxy_call(awn_proxy, "SetInfoByName", &error,
+			G_TYPE_STRING, "claws-mail",
+			G_TYPE_STRING, buf,
+			G_TYPE_INVALID, G_TYPE_INVALID);
+		g_free(buf);
+		
+	} else {
+		dbus_g_proxy_call(awn_proxy, "UnsetInfoByName", &error, G_TYPE_STRING,
+			"claws-mail", G_TYPE_INVALID, G_TYPE_INVALID);
+	}
+	if (error) {
+		debug_print("%s\n", error->message);
+		g_error_free(error);
+	}
+}
+
+static gboolean dbus_status_update_folder_hook(gpointer source, gpointer data)
+{
+	FolderUpdateData *hookdata;
+	hookdata = source;
+	if (hookdata->update_flags & FOLDER_REMOVE_FOLDERITEM)
+		dbus_update(hookdata->item);
+	else
+		dbus_update(NULL);
+
+	return FALSE;
+}
+
+static gboolean dbus_status_update_item_hook(gpointer source, gpointer data)
+{
+	dbus_update(NULL);
+
+	return FALSE;
+}
+
+static void install_dbus_status_handler(void)
+{
+	GError *tmp_error = NULL;
+	DBusGConnection *connection = dbus_g_bus_get(DBUS_BUS_SESSION, &tmp_error);
+	
+	if(!connection) {
+		/* If calling code doesn't do error checking, at least print some debug */
+		debug_print("Failed to open connection to session bus: %s\n",
+				 tmp_error->message);
+		g_error_free(tmp_error);
+		return;
+	}
+	awn_proxy = dbus_g_proxy_new_for_name(connection,
+			"com.google.code.Awn",
+			"/com/google/code/Awn",
+			"com.google.code.Awn");
+	dbus_item_hook_id = hooks_register_hook (FOLDER_ITEM_UPDATE_HOOKLIST, dbus_status_update_item_hook, NULL);
+	if (dbus_item_hook_id == -1) {
+		g_warning(_("Failed to register folder item update hook"));
+		uninstall_dbus_status_handler();
+		return;
+	}
+
+	dbus_folder_hook_id = hooks_register_hook (FOLDER_UPDATE_HOOKLIST, dbus_status_update_folder_hook, NULL);
+	if (dbus_folder_hook_id == -1) {
+		g_warning(_("Failed to register folder update hook"));
+		uninstall_dbus_status_handler();
+		return;
+	}
+}
+#endif
+
 int main(int argc, char *argv[])
 {
 #ifdef MAEMO
 	osso_context_t *osso_context;
 	osso_return_t result;
 #endif
-#ifdef HAVE_NETWORKMANAGER_SUPPORT
+#ifdef HAVE_DBUS_GLIB
 	DBusGConnection *connection;
-  GError *error;
-  DBusGProxy *proxy;
+	GError *error;
+#endif
+#ifdef HAVE_NETWORKMANAGER_SUPPORT
+	DBusGProxy *nm_proxy;
 #endif
 	gchar *userrc;
 	MainWindow *mainwin;
@@ -934,9 +1049,11 @@ int main(int argc, char *argv[])
 	gint num_folder_class = 0;
 	gboolean asked_for_migration = FALSE;
 	gboolean start_done = TRUE;
-
+	GtkUIManager *gui_manager = NULL;
+	GSList *plug_list = NULL;
+	gboolean never_ran = FALSE;
 	START_TIMING("startup");
-	
+
 	sc_starting = TRUE;
 
 #ifdef G_OS_WIN32
@@ -977,7 +1094,6 @@ int main(int argc, char *argv[])
 	sock_init();
 
 	/* check and create unix domain socket for remote operation */
-#ifdef G_OS_UNIX
 	lock_socket = prohibit_duplicate_launch();
 	if (lock_socket < 0) {
 #ifdef HAVE_STARTUP_NOTIFICATION
@@ -987,7 +1103,7 @@ int main(int argc, char *argv[])
 		return 0;
 	}
 
-	if (cmd.status || cmd.status_full) {
+	if (cmd.status || cmd.status_full || cmd.search) {
 		puts("0 Claws Mail not running.");
 		lock_socket_remove();
 		return 0;
@@ -995,32 +1111,52 @@ int main(int argc, char *argv[])
 	
 	if (cmd.exit)
 		return 0;
-#endif
 	if (!g_thread_supported())
 		g_thread_init(NULL);
 
 	gtk_set_locale();
 	gtk_init(&argc, &argv);
 
+#ifdef G_OS_WIN32
+	gtk_settings_set_string_property(gtk_settings_get_default(),
+			"gtk-theme-name",
+			"MS-Windows",
+			"XProperty");
+#endif
+
 #ifdef HAVE_NETWORKMANAGER_SUPPORT
 	went_offline_nm = FALSE;
+	nm_proxy = NULL;
+#endif
+#ifdef HAVE_DBUS_GLIB
 	error = NULL;
-	proxy = NULL;
-  connection = dbus_g_bus_get(DBUS_BUS_SYSTEM, &error);
+	connection = dbus_g_bus_get(DBUS_BUS_SYSTEM, &error);
 
-  if(!connection) {
+	if(!connection) {
 		debug_print("Failed to open connection to system bus: %s\n", error->message);
 		g_error_free(error);
 	}
 	else {
-		proxy = dbus_g_proxy_new_for_name(connection,
-																			"org.freedesktop.NetworkManager",
-																			"/org/freedesktop/NetworkManager",
-																			"org.freedesktop.NetworkManager");
-		dbus_g_proxy_add_signal(proxy,"StateChange", G_TYPE_UINT, G_TYPE_INVALID);
-		dbus_g_proxy_connect_signal(proxy, "StateChange",
-			G_CALLBACK(networkmanager_state_change_cb),
-			NULL,NULL);
+#ifdef HAVE_NETWORKMANAGER_SUPPORT
+		nm_proxy = dbus_g_proxy_new_for_name(connection,
+			"org.freedesktop.NetworkManager",
+			"/org/freedesktop/NetworkManager",
+			"org.freedesktop.NetworkManager");
+		if (nm_proxy) {
+#if NM_CHECK_VERSION(0,8,992)
+			dbus_g_proxy_add_signal(nm_proxy, "StateChanged", G_TYPE_UINT, G_TYPE_INVALID);
+			dbus_g_proxy_connect_signal(nm_proxy, "StateChanged",
+				G_CALLBACK(networkmanager_state_change_cb),
+				NULL,NULL);
+#else
+			dbus_g_proxy_add_signal(nm_proxy, "StateChange", G_TYPE_UINT, G_TYPE_INVALID);
+			dbus_g_proxy_connect_signal(nm_proxy, "StateChange",
+				G_CALLBACK(networkmanager_state_change_cb),
+				NULL,NULL);
+#endif
+		}
+#endif
+		install_dbus_status_handler();
 	}
 #endif
 
@@ -1033,9 +1169,14 @@ int main(int argc, char *argv[])
 	hildon_program = HILDON_PROGRAM(hildon_program_get_instance());
 	static_osso_context = osso_context;
 #endif	
-	gdk_rgb_init();
-	gtk_widget_set_default_colormap(gdk_rgb_get_colormap());
-	gtk_widget_set_default_visual(gdk_rgb_get_visual());
+	gtk_widget_set_default_colormap(
+		gdk_screen_get_system_colormap(
+			gdk_screen_get_default()));
+
+	gui_manager = gtkut_create_ui_manager();
+
+	/* Create container for all the menus we will be adding */
+	MENUITEM_ADDUI("/", "Menus", NULL, GTK_UI_MANAGER_MENUBAR);
 
 	if (!g_thread_supported()) {
 		g_error(_("g_thread is not supported by glib.\n"));
@@ -1100,12 +1241,20 @@ int main(int argc, char *argv[])
 		/* If migration failed or the user didn't want to do it,
 		 * we create a new one (and we'll hit wizard later). 
 		 */
-		if (r == FALSE && !is_dir_exist(RC_DIR) && make_dir(RC_DIR) < 0) {
-#ifdef G_OS_WIN32
-			win32_close_log();
+		if (r == FALSE && !is_dir_exist(RC_DIR)) {
+#ifdef G_OS_UNIX
+			if (copy_dir(SYSCONFDIR "/skel/.claws-mail", RC_DIR) < 0) {
 #endif
-			exit(1);
-	}
+				if (!is_dir_exist(RC_DIR) && make_dir(RC_DIR) < 0) {
+#ifdef G_OS_WIN32
+					win32_close_log();
+#endif
+					exit(1);
+				}
+#ifdef G_OS_UNIX
+			}
+#endif
+		}
 	}
 	
 
@@ -1185,7 +1334,7 @@ int main(int argc, char *argv[])
 	prefs_receive_init();
 	prefs_send_init();
 	tags_read_tags();
-#ifdef USE_ASPELL
+#ifdef USE_ENCHANT
 	gtkaspell_checkers_init();
 	prefs_spelling_init();
 #endif
@@ -1209,7 +1358,7 @@ int main(int argc, char *argv[])
 	mainwin = main_window_create();
 
 #ifdef HAVE_NETWORKMANAGER_SUPPORT
-		networkmanager_state_change_cb(proxy,NULL,mainwin);
+	networkmanager_state_change_cb(nm_proxy,NULL,mainwin);
 #endif
 
 #ifdef MAEMO
@@ -1242,21 +1391,24 @@ int main(int argc, char *argv[])
 	manage_window_focus_in(mainwin->window, NULL, NULL);
 	folderview = mainwin->folderview;
 
-	gtk_clist_freeze(GTK_CLIST(mainwin->folderview->ctree));
+	gtk_cmclist_freeze(GTK_CMCLIST(mainwin->folderview->ctree));
 	folder_item_update_freeze();
 
 	/* register the callback of unix domain socket input */
-#ifdef G_OS_UNIX
-	lock_socket_tag = gdk_input_add(lock_socket,
+	lock_socket_tag = claws_input_add(lock_socket,
 					GDK_INPUT_READ | GDK_INPUT_EXCEPTION,
 					lock_socket_input_cb,
-					mainwin);
-#endif
+					mainwin, TRUE);
 
 
 	prefs_account_init();
 	account_read_config_all();
 
+#ifdef HAVE_LIBETPAN
+	imap_main_init(prefs_common.skip_ssl_cert_check);
+	imap_main_set_timeout(prefs_common.io_timeout_secs);
+	nntp_main_init(prefs_common.skip_ssl_cert_check);
+#endif	
 	/* If we can't read a folder list or don't have accounts,
 	 * it means the configuration's not done. Either this is
 	 * a brand new install, either a failed/refused migration.
@@ -1280,6 +1432,7 @@ int main(int argc, char *argv[])
 		}
 		main_window_reflect_prefs_all_now();
 		folder_write_list();
+		never_ran = TRUE;
 	}
 
 	if (!account_get_list()) {
@@ -1299,6 +1452,7 @@ int main(int argc, char *argv[])
 #endif
 			exit(1);
 		}
+		never_ran = TRUE;
 	}
 
 	
@@ -1310,20 +1464,12 @@ int main(int argc, char *argv[])
 	if (claws_crashed())
 		main_window_popup(mainwin);
 
-#ifdef USE_GNUTLS
-	gcry_control (GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
-#endif
-
-#ifdef HAVE_LIBETPAN
-	imap_main_init(prefs_common.skip_ssl_cert_check);
-	imap_main_set_timeout(prefs_common.io_timeout_secs);
-	nntp_main_init(prefs_common.skip_ssl_cert_check);
-#endif	
 	account_set_missing_folder();
 	folder_set_missing_folders();
 	folderview_set(folderview);
 
 	prefs_matcher_read_config();
+	quicksearch_set_search_strings(mainwin->summaryview->quicksearch);
 
 	/* make one all-folder processing before using claws */
 	main_window_cursor_wait(mainwin);
@@ -1376,16 +1522,39 @@ int main(int argc, char *argv[])
 		prefs_matcher_read_config();
 	}
 	
-	if (plugin_get_unloaded_list() != NULL) {
+	if ((plug_list = plugin_get_unloaded_list()) != NULL) {
+		GSList *cur;
+		gchar *list = NULL;
+		gint num_plugins = 0;
+		for (cur = plug_list; cur; cur = cur->next) {
+			Plugin *plugin = (Plugin *)cur->data;
+			gchar *tmp = g_strdup_printf("%s\n%s",
+				list? list:"",
+				plugin_get_name(plugin));
+			g_free(list);
+			list = tmp;
+			num_plugins++;
+		}
 		main_window_cursor_normal(mainwin);
-		alertpanel_warning(_("Some plugin(s) failed to load. "
+		main_window_popup(mainwin);
+		alertpanel_warning(ngettext(
+				     "The following plugin failed to load. "
 				     "Check the Plugins configuration "
-				     "for more information."));
+				     "for more information:\n%s",
+				     "The following plugins failed to load. "
+				     "Check the Plugins configuration "
+				     "for more information:\n%s", 
+				     num_plugins), 
+				     list);
 		main_window_cursor_wait(mainwin);
+		g_free(list);
+		g_slist_free(plug_list);
 	}
 
- 	plugin_load_standard_plugins ();
-      
+	if (never_ran) {
+		prefs_common_write_config();
+	 	plugin_load_standard_plugins ();
+	}
 	/* if not crashed, show window now */
 	if (!claws_crashed()) {
 		/* apart if something told not to show */
@@ -1456,7 +1625,7 @@ int main(int argc, char *argv[])
 #endif
 
 	folder_item_update_thaw();
-	gtk_clist_thaw(GTK_CLIST(mainwin->folderview->ctree));
+	gtk_cmclist_thaw(GTK_CMCLIST(mainwin->folderview->ctree));
 	main_window_cursor_normal(mainwin);
 
 	if (!cmd.target && prefs_common.goto_last_folder_on_startup &&
@@ -1515,8 +1684,11 @@ int main(int argc, char *argv[])
 	osso_deinitialize(osso_context);
 #endif
 #ifdef HAVE_NETWORKMANAGER_SUPPORT
-	if(proxy)
-		g_object_unref(proxy);
+	if(nm_proxy)
+		g_object_unref(nm_proxy);
+#endif
+#ifdef HAVE_DBUS_GLIB
+	uninstall_dbus_status_handler();
 	if(connection)
 		dbus_g_connection_unref(connection);
 #endif
@@ -1545,16 +1717,13 @@ static void save_all_caches(FolderItem *item, gpointer data)
 static void exit_claws(MainWindow *mainwin)
 {
 	gchar *filename;
+	gboolean have_connectivity;
 
 	sc_exiting = TRUE;
 
 	debug_print("shutting down\n");
 	inc_autocheck_timer_remove();
 
-	if (prefs_common.clean_on_exit && !emergency_exit) {
-		main_window_empty_trash(mainwin, prefs_common.ask_on_clean);
-	}
-
 #ifdef HAVE_NETWORKMANAGER_SUPPORT
 	if (prefs_common.work_offline && went_offline_nm)
 		prefs_common.work_offline = FALSE;
@@ -1564,7 +1733,7 @@ static void exit_claws(MainWindow *mainwin)
 	if(mainwin->folderview->opened) {
 		FolderItem *item;
 
-		item = gtk_ctree_node_get_row_data(GTK_CTREE(mainwin->folderview->ctree), mainwin->folderview->opened);
+		item = gtk_cmctree_node_get_row_data(GTK_CMCTREE(mainwin->folderview->ctree), mainwin->folderview->opened);
 		summary_save_prefs_to_folderitem(mainwin->folderview->summaryview, item);
 		prefs_common.last_opened_folder = folder_item_get_identifier(item);
 	}
@@ -1591,13 +1760,18 @@ static void exit_claws(MainWindow *mainwin)
 	close_log_file(LOG_PROTOCOL);
 	close_log_file(LOG_DEBUG_FILTERING);
 
+#ifdef HAVE_NETWORKMANAGER_SUPPORT
+	have_connectivity = networkmanager_is_online(NULL); 
+#else
+	have_connectivity = TRUE;
+#endif
 #ifdef HAVE_LIBETPAN
-	imap_main_done();
-	nntp_main_done();
+	imap_main_done(have_connectivity);
+	nntp_main_done(have_connectivity);
 #endif
 	/* delete crashfile */
 	if (!cmd.crash)
-		g_unlink(get_crashfile_name());
+		claws_unlink(get_crashfile_name());
 
 	lock_socket_remove();
 
@@ -1631,7 +1805,7 @@ static void exit_claws(MainWindow *mainwin)
 	prefs_logging_done();
 	prefs_send_done();
 	tags_write_tags();
-#ifdef USE_ASPELL       
+#ifdef USE_ENCHANT       
 	prefs_spelling_done();
 	gtkaspell_checkers_quit();
 #endif
@@ -1649,7 +1823,7 @@ static void parse_cmd_opt(int argc, char *argv[])
 		} else if (!strncmp(argv[i], "--receive", 9)) {
 			cmd.receive = TRUE;
 		} else if (!strncmp(argv[i], "--compose", 9)) {
-			const gchar *p = argv[i + 1];
+			const gchar *p = (i+1 < argc)?argv[i+1]:NULL;
 
 			cmd.compose = TRUE;
 			cmd.compose_mailto = NULL;
@@ -1662,13 +1836,13 @@ static void parse_cmd_opt(int argc, char *argv[])
 				i++;
 			}
 		} else if (!strncmp(argv[i], "--subscribe", 11)) {
-			const gchar *p = argv[i + 1];
+			const gchar *p = (i+1 < argc)?argv[i+1]:NULL;
 			if (p && *p != '\0' && *p != '-') {
 				cmd.subscribe = TRUE;
 				cmd.subscribe_uri = p;
 			}
 		} else if (!strncmp(argv[i], "--attach", 8)) {
-			const gchar *p = argv[i + 1];
+			const gchar *p = (i+1 < argc)?argv[i+1]:NULL;
 			gchar *file = NULL;
 
 			while (p && *p != '\0' && *p != '-') {
@@ -1690,7 +1864,7 @@ static void parse_cmd_opt(int argc, char *argv[])
 				}
 				g_ptr_array_add(cmd.attach_files, file);
 				i++;
-				p = argv[i + 1];
+				p = (i+1 < argc)?argv[i+1]:NULL;
 			}
 		} else if (!strncmp(argv[i], "--send", 6)) {
 			cmd.send = TRUE;
@@ -1704,7 +1878,7 @@ static void parse_cmd_opt(int argc, char *argv[])
 			g_print("Claws Mail version " VERSION "\n");
 			exit(0);
  		} else if (!strncmp(argv[i], "--status-full", 13)) {
- 			const gchar *p = argv[i + 1];
+ 			const gchar *p = (i+1 < argc)?argv[i+1]:NULL;
  
  			cmd.status_full = TRUE;
  			while (p && *p != '\0' && *p != '-') {
@@ -1715,10 +1889,10 @@ static void parse_cmd_opt(int argc, char *argv[])
  				g_ptr_array_add(cmd.status_full_folders,
  						g_strdup(p));
  				i++;
- 				p = argv[i + 1];
+ 				p = (i+1 < argc)?argv[i+1]:NULL;
  			}
   		} else if (!strncmp(argv[i], "--status", 8)) {
- 			const gchar *p = argv[i + 1];
+ 			const gchar *p = (i+1 < argc)?argv[i+1]:NULL;
  
   			cmd.status = TRUE;
  			while (p && *p != '\0' && *p != '-') {
@@ -1727,8 +1901,18 @@ static void parse_cmd_opt(int argc, char *argv[])
  				g_ptr_array_add(cmd.status_folders,
  						g_strdup(p));
  				i++;
- 				p = argv[i + 1];
+ 				p = (i+1 < argc)?argv[i+1]:NULL;
  			}
+		} else if (!strncmp(argv[i], "--search", 8)) {
+			cmd.search_folder    = (i+1 < argc)?argv[i+1]:NULL;
+			cmd.search_type      = (i+2 < argc)?argv[i+2]:NULL;
+			cmd.search_request   = (i+3 < argc)?argv[i+3]:NULL;
+			const char* rec      = (i+4 < argc)?argv[i+4]:NULL;
+			cmd.search_recursive = TRUE;
+			if (rec && (tolower(*rec)=='n' || tolower(*rec)=='f' || *rec=='0'))
+				cmd.search_recursive = FALSE;
+			if (cmd.search_folder && cmd.search_type && cmd.search_request)
+				cmd.search = TRUE;
 		} else if (!strncmp(argv[i], "--online", 8)) {
 			cmd.online_mode = ONLINE_MODE_ONLINE;
 		} else if (!strncmp(argv[i], "--offline", 9)) {
@@ -1745,6 +1929,13 @@ static void parse_cmd_opt(int argc, char *argv[])
 			          "                         attached"));
 			g_print("%s\n", _("  --receive              receive new messages"));
 			g_print("%s\n", _("  --receive-all          receive new messages of all accounts"));
+			g_print("%s\n", _("  --search folder type request [recursive]"));
+			g_print("%s\n", _("                         searches mail"));
+			g_print("%s\n", _("                         folder ex.: \"#mh/Mailbox/inbox\" or \"Mail\""));
+			g_print("%s\n", _("                         type: s[ubject],f[rom],t[o],e[xtended],m[ixed] or g: tag"));
+			g_print("%s\n", _("                         request: search string"));
+			g_print("%s\n", _("                         recursive: false if arg. starts with 0, n, N, f or F"));
+
 			g_print("%s\n", _("  --send                 send all queued messages"));
  			g_print("%s\n", _("  --status [folder]...   show the total number of messages"));
  			g_print("%s\n", _("  --status-full [folder]...\n"
@@ -1766,7 +1957,7 @@ static void parse_cmd_opt(int argc, char *argv[])
 			exit(1);
 		} else if (!strncmp(argv[i], "--crash", 7)) {
 			cmd.crash = TRUE;
-			cmd.crash_params = g_strdup(argv[i + 1]);
+			cmd.crash_params = g_strdup((i+1 < argc)?argv[i+1]:NULL);
 			i++;
 		} else if (!strncmp(argv[i], "--config-dir", sizeof "--config-dir" - 1)) {
 			g_print(RC_DIR "\n");
@@ -1781,7 +1972,7 @@ static void parse_cmd_opt(int argc, char *argv[])
 			cmd.target = argv[i+1];
 		} else if (i == 1 && argc == 2) {
 			/* only one parameter. Do something intelligent about it */
-			if (strstr(argv[i], "@") && !strstr(argv[i], "://")) {
+			if ((strstr(argv[i], "@")||!strncmp(argv[i], "mailto:", 7)) && !strstr(argv[i], "://")) {
 				const gchar *p = argv[i];
 
 				cmd.compose = TRUE;
@@ -1826,7 +2017,7 @@ static void initial_processing(FolderItem *item, gpointer data)
 	MainWindow *mainwin = (MainWindow *)data;
 	gchar *buf;
 
-	g_return_if_fail(item);
+	cm_return_if_fail(item);
 	buf = g_strdup_printf(_("Processing (%s)..."), 
 			      item->path 
 			      ? item->path 
@@ -1894,6 +2085,10 @@ void app_will_exit(GtkWidget *widget, gpointer data)
 {
 	MainWindow *mainwin = data;
 	
+	if (gtk_main_level() == 0) {
+		debug_print("not even started\n");
+		return;
+	}
 	if (sc_exiting == TRUE) {
 		debug_print("exit pending\n");
 		return;
@@ -1982,6 +2177,7 @@ static gchar *get_crashfile_name(void)
 static gint prohibit_duplicate_launch(void)
 {
 	gint uxsock;
+#ifdef G_OS_UNIX
 	gchar *path;
 
 	path = claws_get_socket_name();
@@ -1991,10 +2187,28 @@ static gint prohibit_duplicate_launch(void)
 		x_display = g_strdup(g_getenv("DISPLAY"));
 
 	if (uxsock < 0) {
-		g_unlink(path);
+		claws_unlink(path);
 		return fd_open_unix(path);
 	}
+#else
+        HANDLE hmutex;
 
+        hmutex = CreateMutexA(NULL, FALSE, "ClawsMail");
+        if (!hmutex) {
+                debug_print("cannot create Mutex\n");
+                return -1;
+        }
+        if (GetLastError() != ERROR_ALREADY_EXISTS) {
+                uxsock = fd_open_inet(50216);
+                if (uxsock < 0)
+                        return 0;
+                return uxsock;
+        }
+
+        uxsock = fd_connect_inet(50216);
+        if (uxsock < 0)
+                return -1;
+#endif
 	/* remote command mode */
 
 	debug_print("another Claws Mail instance is already running.\n");
@@ -2075,7 +2289,21 @@ static gint prohibit_duplicate_launch(void)
 		gchar *str = g_strdup_printf("select %s\n", cmd.target);
 		fd_write_all(uxsock, str, strlen(str));
 		g_free(str);
+	} else if (cmd.search) {
+		gchar buf[BUFFSIZE];
+		gchar *str =
+			g_strdup_printf("search %s\n%s\n%s\n%c\n",
+							cmd.search_folder, cmd.search_type, cmd.search_request,
+							(cmd.search_recursive==TRUE)?'1':'0');
+		fd_write_all(uxsock, str, strlen(str));
+		g_free(str);
+		for (;;) {
+			fd_gets(uxsock, buf, sizeof(buf));
+			if (!strncmp(buf, ".\n", 2)) break;
+			fputs(buf, stdout);
+		}
 	} else {
+#ifndef G_OS_WIN32
 		gchar buf[BUFSIZ];
 		fd_write_all(uxsock, "get_display\n", 12);
 		memset(buf, 0, sizeof(buf));
@@ -2088,6 +2316,9 @@ static gint prohibit_duplicate_launch(void)
 			uxsock = fd_connect_unix(path);
 			fd_write_all(uxsock, "popup\n", 6);
 		}
+#else
+		fd_write_all(uxsock, "popup\n", 6);
+#endif
 	}
 
 	fd_close(uxsock);
@@ -2098,17 +2329,19 @@ static gint lock_socket_remove(void)
 {
 #ifdef G_OS_UNIX
 	gchar *filename;
-
+#endif
 	if (lock_socket < 0) {
 		return -1;
 	}
 
 	if (lock_socket_tag > 0) {
-		gdk_input_remove(lock_socket_tag);
+		g_source_remove(lock_socket_tag);
 	}
 	fd_close(lock_socket);
+
+#ifdef G_OS_UNIX
 	filename = claws_get_socket_name();
-	g_unlink(filename);
+	claws_unlink(filename);
 #endif
 
 	return 0;
@@ -2147,6 +2380,8 @@ static void lock_socket_input_cb(gpointer data,
 	MainWindow *mainwin = (MainWindow *)data;
 	gint sock;
 	gchar buf[BUFFSIZE];
+	/* re-use the same quicksearch (& avoid matcher_list mem.leaks) */
+	static QuickSearch *quicksearch = NULL;
 
 	sock = fd_accept(source);
 	fd_gets(sock, buf, sizeof(buf));
@@ -2167,10 +2402,9 @@ static void lock_socket_input_cb(gpointer data,
 		mailto = g_strdup(buf + strlen("compose_attach") + 1);
 		files = g_ptr_array_new();
 		while (fd_gets(sock, buf, sizeof(buf)) > 0) {
-			if (buf[0] == '.' && buf[1] == '\n') {
-				break;
-			}
 			strretchomp(buf);
+			if (!strcmp2(buf, "."))
+				break;
 			g_ptr_array_add(files, g_strdup(buf));
 		}
 		open_compose_new(mailto, files);
@@ -2203,11 +2437,80 @@ static void lock_socket_input_cb(gpointer data,
 	} else if (!strncmp(buf, "select ", 7)) {
 		const gchar *target = buf+7;
 		mainwindow_jump_to(target, TRUE);
+	} else if (!strncmp(buf, "search ", 7)) {
+		FolderItem* folderItem = NULL;
+		GSList *messages = NULL;
+		gchar *folder_name, *request;
+		QuickSearchType searchType = QUICK_SEARCH_EXTENDED;
+		gboolean recursive;
+
+		if (quicksearch==NULL)
+			quicksearch = quicksearch_new_nogui();
+		
+		folder_name = g_strdup(buf+7);
+		strretchomp(folder_name);
+
+		if (fd_gets(sock, buf, sizeof(buf)) <= 0) {
+			g_free(folder_name);
+			folder_name=NULL;
+		}
+		searchType = quicksearch_type(buf);
+		if (fd_gets(sock, buf, sizeof(buf)) <= 0) {
+			g_free(folder_name);
+			folder_name=NULL;
+		}
+		request = g_strdup(buf);
+		strretchomp(request);
+
+		recursive = TRUE;
+		if (fd_gets(sock, buf, sizeof(buf)) > 0)
+			if (buf[0]=='0')
+				recursive = FALSE;
+
+		debug_print("search: %s %i %s %i\n",folder_name,searchType,request,recursive);
+
+		if (folder_name)
+			folderItem = folder_find_item_from_identifier(folder_name);
+		if (folder_name && folderItem == NULL) {
+			debug_print("Unknow folder item : '%s', searching folder\n",folder_name);
+			Folder* folder = folder_find_from_path(folder_name);
+			if (folder != NULL)
+				folderItem = FOLDER_ITEM(folder->node->data);
+			else
+				debug_print("Unknown folder: '%s'\n",folder_name);
+		} else {
+			debug_print("%s %s\n",folderItem->name, folderItem->path);
+        }
+		if (folderItem != NULL) {
+			quicksearch_set(quicksearch, searchType, request);
+			quicksearch_set_recursive(quicksearch, recursive);
+			search_msgs_in_folders(&messages, quicksearch, folderItem);
+		} else {
+			g_print("Folder '%s' not found.\n'", folder_name);
+		}
+
+		GSList *cur;
+		for (cur=messages; cur != NULL; cur = cur->next) {
+			MsgInfo* msg = (MsgInfo *)cur->data;
+			gchar *file = procmsg_get_message_file_path(msg);
+			fd_write_all(sock, file, strlen(file));
+			fd_write_all(sock, "\n", 1);
+			g_free(file);
+		}
+		fd_write_all(sock, ".\n", 2);
+
+		if (messages != NULL)
+			procmsg_msg_list_free(messages);
+		g_free(folder_name);
+		g_free(request);
 	} else if (!strncmp(buf, "exit", 4)) {
+		if (prefs_common.clean_on_exit && !prefs_common.ask_on_clean) {
+			procmsg_empty_all_trash();
+                }
 		app_will_exit(NULL, mainwin);
 	}
-
 	fd_close(sock);
+
 }
 
 static void open_compose_new(const gchar *address, GPtrArray *attach_files)
@@ -2320,7 +2623,7 @@ static void install_memory_sighandler()
 	struct sigaction act;
 	int flags;
 
-	mem_notify_fd = open("/dev/mem_notify", O_RDONLY|O_NONBLOCK);
+	mem_notify_fd = g_open("/dev/mem_notify", O_RDONLY|O_NONBLOCK, 0);
 	if (mem_notify_fd == -1) {
 		debug_print("/dev/mem_notify not available (%s)\n", 
 			strerror(errno));
@@ -2406,7 +2709,7 @@ gboolean networkmanager_is_online(GError **error)
 {
 	DBusGConnection *connection;
 	DBusGProxy *proxy;
-	GError *tmp_error;
+	GError *tmp_error = NULL;
 	gboolean retVal;
 	guint32 state;
 
@@ -2447,7 +2750,14 @@ gboolean networkmanager_is_online(GError **error)
 		g_propagate_error(error, tmp_error);
 		return TRUE;
 	}
-
-	return (state == NM_STATE_CONNECTED || state == NM_STATE_UNKNOWN);
+#if NM_CHECK_VERSION(0,8,992)
+    	return (state == NM_STATE_CONNECTED_LOCAL ||
+		state == NM_STATE_CONNECTED_SITE ||
+		state == NM_STATE_CONNECTED_GLOBAL ||
+		state == NM_STATE_UNKNOWN);
+#else
+    	return (state == NM_STATE_CONNECTED ||
+		state == NM_STATE_UNKNOWN);
+#endif
 }
 #endif
