@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 1999-2011 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <gtk/gtkoptionmenu.h>
 #include <gdk/gdkkeysyms.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -100,29 +99,29 @@ static void prefs_filtering_set_list	(void);
 static gboolean prefs_filtering_search_func_cb (GtkTreeModel *model, gint column, 
 						const gchar *key, GtkTreeIter *iter, 
 						gpointer search_data);
-static void prefs_filtering_register_cb	(void);
-static void prefs_filtering_substitute_cb	(void);
-static void prefs_filtering_delete_cb	(void);
-static void prefs_filtering_delete_all_cb(void);
-static void prefs_filtering_clear_cb(void);
-static void prefs_filtering_duplicate_cb(void);
-static void prefs_filtering_top		(void);
-static void prefs_filtering_page_up(void);
-static void prefs_filtering_up		(void);
-static void prefs_filtering_down	(void);
-static void prefs_filtering_page_down(void);
-static void prefs_filtering_bottom	(void);
+static void prefs_filtering_register_cb	(gpointer action, gpointer data);
+static void prefs_filtering_substitute_cb	(gpointer action, gpointer data);
+static void prefs_filtering_delete_cb	(gpointer action, gpointer data);
+static void prefs_filtering_delete_all_cb(gpointer action, gpointer data);
+static void prefs_filtering_clear_cb(gpointer action, gpointer data);
+static void prefs_filtering_duplicate_cb(gpointer action, gpointer data);
+static void prefs_filtering_top		(gpointer action, gpointer data);
+static void prefs_filtering_page_up	(gpointer action, gpointer data);
+static void prefs_filtering_up		(gpointer action, gpointer data);
+static void prefs_filtering_down	(gpointer action, gpointer data);
+static void prefs_filtering_page_down	(gpointer action, gpointer data);
+static void prefs_filtering_bottom	(gpointer action, gpointer data);
 static gint prefs_filtering_deleted	(GtkWidget	*widget,
 					 GdkEventAny	*event,
 					 gpointer	 data);
 static gboolean prefs_filtering_key_pressed(GtkWidget	*widget,
 					 GdkEventKey	*event,
 					 gpointer	 data);
-static void prefs_filtering_cancel	(void);
-static void prefs_filtering_ok		(void);
+static void prefs_filtering_cancel	(gpointer action, gpointer data);
+static void prefs_filtering_ok		(gpointer action, gpointer data);
 
-static void prefs_filtering_condition_define	(void);
-static void prefs_filtering_action_define(void);
+static void prefs_filtering_condition_define	(gpointer action, gpointer data);
+static void prefs_filtering_action_define	(gpointer action, gpointer data);
 static gint prefs_filtering_list_view_set_row	(gint row, FilteringProp * prop);
 					  
 static void prefs_filtering_reset_dialog	(void);
@@ -221,6 +220,7 @@ void prefs_filtering_open(GSList ** p_processing,
 	}
 
 	gtk_widget_show(filtering.window);
+	gtk_window_set_modal(GTK_WINDOW(filtering.window), TRUE);
 
 	start_address_completion(NULL);
 }
@@ -228,7 +228,7 @@ void prefs_filtering_open(GSList ** p_processing,
 static void prefs_filtering_size_allocate_cb(GtkWidget *widget,
 					 GtkAllocation *allocation)
 {
-	g_return_if_fail(allocation != NULL);
+	cm_return_if_fail(allocation != NULL);
 
 	prefs_common.filteringwin_width = allocation->width;
 	prefs_common.filteringwin_height = allocation->height;
@@ -245,6 +245,7 @@ static void prefs_filtering_close(void)
 				(filtering.cond_list_view)));
 	gtk_list_store_clear(store);
 	gtk_widget_hide(filtering.window);
+	gtk_window_set_modal(GTK_WINDOW(filtering.window), FALSE);
 	inc_unlock();
 }
 
@@ -255,9 +256,10 @@ static void prefs_filtering_account_option_menu_populate(void)
 
 	accounts = account_get_list();
 
-	g_return_if_fail(accounts != NULL);
+	cm_return_if_fail(accounts != NULL);
 
 	COMBOBOX_ADD(filtering.account_combobox_list, Q_("Filtering Account Menu|All"), 0);
+	COMBOBOX_ADD(filtering.account_combobox_list, NULL, 0);
 	for (; accounts != NULL; accounts = accounts->next) {
 		PrefsAccount *ac = (PrefsAccount *)accounts->data;
 
@@ -321,23 +323,20 @@ static void prefs_filtering_create(void)
 	GtkWidget *top_btn;
 	GtkWidget *up_btn;
 	GtkWidget *down_btn;
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	GtkWidget *page_up_btn;
 	GtkWidget *page_down_btn;
 #endif
 	GtkWidget *bottom_btn;
 	GtkWidget *table;
 	static GdkGeometry geometry;
-	GtkTooltips *tooltips;
+	CLAWS_TIP_DECL();
 
 	debug_print("Creating filtering configuration window...\n");
 
-	tooltips = gtk_tooltips_new();
-
 	window = gtkut_window_new(GTK_WINDOW_TOPLEVEL, "prefs_filtering");
 	gtk_container_set_border_width (GTK_CONTAINER (window), 8);
 	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
-	gtk_window_set_modal (GTK_WINDOW (window), TRUE);
 	gtk_window_set_resizable(GTK_WINDOW (window), TRUE);
 
 	vbox = gtk_vbox_new (FALSE, 6);
@@ -468,8 +467,8 @@ static void prefs_filtering_create(void)
 	gtk_box_pack_start (GTK_BOX (btn_hbox), reg_btn, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT (reg_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_register_cb), NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), reg_btn,
-			_("Append the new rule above to the list"), NULL);
+	CLAWS_SET_TIP(reg_btn,
+			_("Append the new rule above to the list"));
 
 	subst_btn = gtkut_get_replace_btn (_("Replace"));
 	gtk_widget_show (subst_btn);
@@ -477,24 +476,24 @@ static void prefs_filtering_create(void)
 	g_signal_connect(G_OBJECT (subst_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_substitute_cb),
 			 NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), subst_btn,
-			_("Replace the selected rule in list with the rule above"), NULL);
+	CLAWS_SET_TIP(subst_btn,
+			_("Replace the selected rule in list with the rule above"));
 
 	del_btn = gtk_button_new_from_stock (GTK_STOCK_DELETE);
 	gtk_widget_show (del_btn);
 	gtk_box_pack_start (GTK_BOX (btn_hbox), del_btn, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT (del_btn), "clicked",
 			G_CALLBACK(prefs_filtering_delete_cb), NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), del_btn,
-			_("Delete the selected rule from the list"), NULL);
+	CLAWS_SET_TIP(del_btn,
+			_("Delete the selected rule from the list"));
 
 	clear_btn = gtk_button_new_from_stock (GTK_STOCK_CLEAR);
 	gtk_widget_show (clear_btn);
 	gtk_box_pack_start (GTK_BOX (btn_hbox), clear_btn, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT (clear_btn), "clicked",
 			G_CALLBACK(prefs_filtering_clear_cb), NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), clear_btn,
-			_("Clear all the input fields in the dialog"), NULL);
+	CLAWS_SET_TIP(clear_btn,
+			_("Clear all the input fields in the dialog"));
 
 	cond_hbox = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (cond_hbox);
@@ -522,10 +521,10 @@ static void prefs_filtering_create(void)
 	gtk_box_pack_start (GTK_BOX (btn_vbox), top_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT (top_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_top), NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), top_btn,
-			_("Move the selected rule to the top"), NULL);
+	CLAWS_SET_TIP(top_btn,
+			_("Move the selected rule to the top"));
 
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	page_up_btn = gtk_button_new_with_mnemonic (_("Page up"));
 	gtk_button_set_image(GTK_BUTTON(page_up_btn),
 			gtk_image_new_from_stock(GTK_STOCK_GO_UP,GTK_ICON_SIZE_BUTTON));
@@ -533,8 +532,8 @@ static void prefs_filtering_create(void)
 	gtk_box_pack_start (GTK_BOX (btn_vbox), page_up_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT (page_up_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_page_up), NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), page_up_btn,
-			_("Move the selected rule one page up"), NULL);
+	CLAWS_SET_TIP(page_up_btn,
+			_("Move the selected rule one page up"));
 #endif
 
 	up_btn = gtk_button_new_from_stock (GTK_STOCK_GO_UP);
@@ -542,18 +541,18 @@ static void prefs_filtering_create(void)
 	gtk_box_pack_start (GTK_BOX (btn_vbox), up_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT (up_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_up), NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), up_btn,
-			_("Move the selected rule up"), NULL);
+	CLAWS_SET_TIP(up_btn,
+			_("Move the selected rule up"));
 
 	down_btn = gtk_button_new_from_stock (GTK_STOCK_GO_DOWN);
 	gtk_widget_show (down_btn);
 	gtk_box_pack_start (GTK_BOX (btn_vbox), down_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT (down_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_down), NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), down_btn,
-			_("Move the selected rule down"), NULL);
+	CLAWS_SET_TIP(down_btn,
+			_("Move the selected rule down"));
 
-#ifndef MAEMO
+#ifndef GENERIC_UMPC
 	page_down_btn = gtk_button_new_with_mnemonic (_("Page down"));
 	gtk_button_set_image(GTK_BUTTON(page_down_btn),
 			gtk_image_new_from_stock(GTK_STOCK_GO_DOWN,GTK_ICON_SIZE_BUTTON));
@@ -561,8 +560,8 @@ static void prefs_filtering_create(void)
 	gtk_box_pack_start (GTK_BOX (btn_vbox), page_down_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT (page_down_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_page_down), NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), page_down_btn,
-			_("Move the selected rule one page down"), NULL);
+	CLAWS_SET_TIP(page_down_btn,
+			_("Move the selected rule one page down"));
 #endif
 
 	bottom_btn = gtk_button_new_from_stock (GTK_STOCK_GOTO_BOTTOM);
@@ -570,8 +569,8 @@ static void prefs_filtering_create(void)
 	gtk_box_pack_start (GTK_BOX (btn_vbox), bottom_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT (bottom_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_bottom), NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), bottom_btn,
-			_("Move the selected rule to the bottom"), NULL);
+	CLAWS_SET_TIP(bottom_btn,
+			_("Move the selected rule to the bottom"));
 
 	if (!geometry.min_height) {
 		geometry.min_width = 500;
@@ -656,9 +655,13 @@ static void rename_path(GSList * filters,
 	gint oldpathlen;
         GSList * action_cur;
         GSList * cur;
-
+	const gchar *separator=G_DIR_SEPARATOR_S;
+	gboolean matched = FALSE;
+#ifdef G_OS_WIN32
+again:
+#endif
 	oldpathlen = strlen(old_path);
-	old_path_with_sep = g_strconcat(old_path,G_DIR_SEPARATOR_S,NULL);
+	old_path_with_sep = g_strconcat(old_path,separator,NULL);
 
 	for (cur = filters; cur != NULL; cur = cur->next) {
 		FilteringProp   *filtering = (FilteringProp *)cur->data;
@@ -688,28 +691,30 @@ static void rename_path(GSList * filters,
                                         while (*base == G_DIR_SEPARATOR) base++;
                                         if (*base == '\0')
                                                 dest_path = g_strconcat(prefix,
-                                                    G_DIR_SEPARATOR_S,
+                                                    separator,
                                                     new_path, NULL);
                                         else
                                                 dest_path = g_strconcat(prefix,
-                                                    G_DIR_SEPARATOR_S,
+                                                    separator,
                                                     new_path,
-                                                    G_DIR_SEPARATOR_S,
+                                                    separator,
                                                     base, NULL);
                                         
                                         g_free(prefix);
                                         g_free(action->destination);
                                         action->destination = dest_path;
+					matched = TRUE;
                                 } else { /* for non-leaf folders */
                                         /* compare with trailing slash */
                                         if (!strncmp(old_path_with_sep, action->destination, oldpathlen+1)) {
                                                 
                                                 suffix = action->destination + oldpathlen + 1;
                                                 dest_path = g_strconcat(new_path,
-                                                    G_DIR_SEPARATOR_S,
+                                                    separator,
                                                     suffix, NULL);
                                                 g_free(action->destination);
                                                 action->destination = dest_path;
+						matched = TRUE;
                                         }
                                 }
                         } else {
@@ -718,10 +723,19 @@ static void rename_path(GSList * filters,
                                         dest_path = g_strdup(new_path);
                                         g_free(action->destination);
                                         action->destination = dest_path;
+					matched = TRUE;
                                 }
                         }
                 }
         }
+	
+	g_free(old_path_with_sep);
+#ifdef G_OS_WIN32
+	if (!strcmp(separator, G_DIR_SEPARATOR_S) && !matched) {
+		separator = "/";
+		goto again;
+	}
+#endif
 }
 
 static gboolean prefs_filtering_rename_path_func(GNode *node, gpointer data)
@@ -736,9 +750,9 @@ static gboolean prefs_filtering_rename_path_func(GNode *node, gpointer data)
 	old_path = paths[0];
 	new_path = paths[1];
 
-	g_return_val_if_fail(old_path != NULL, FALSE);
-	g_return_val_if_fail(new_path != NULL, FALSE);
-	g_return_val_if_fail(node != NULL, FALSE);
+	cm_return_val_if_fail(old_path != NULL, FALSE);
+	cm_return_val_if_fail(new_path != NULL, FALSE);
+	cm_return_val_if_fail(node != NULL, FALSE);
 
         item = node->data;
         if (!item || !item->prefs)
@@ -789,9 +803,9 @@ static gboolean prefs_filtering_rename_tag_func(GNode *node, gpointer data)
 	old_tag = tags[0];
 	new_tag = tags[1];
 
-	g_return_val_if_fail(old_tag != NULL, FALSE);
-	g_return_val_if_fail(new_tag != NULL, FALSE);
-	g_return_val_if_fail(node != NULL, FALSE);
+	cm_return_val_if_fail(old_tag != NULL, FALSE);
+	cm_return_val_if_fail(new_tag != NULL, FALSE);
+	cm_return_val_if_fail(node != NULL, FALSE);
 
         item = node->data;
         if (!item || !item->prefs)
@@ -876,8 +890,8 @@ static gboolean prefs_filtering_delete_path_func(GNode *node, gpointer data)
 	FolderItem *item;
         GSList ** p_filters;
 	
-	g_return_val_if_fail(path != NULL, FALSE);
-	g_return_val_if_fail(node != NULL, FALSE);
+	cm_return_val_if_fail(path != NULL, FALSE);
+	cm_return_val_if_fail(node != NULL, FALSE);
 
         item = node->data;
         if (!item || !item->prefs)
@@ -971,18 +985,21 @@ static gboolean prefs_filtering_search_func_cb (GtkTreeModel *model, gint column
 						GtkTreeIter *iter, gpointer search_data) 
 {
 	gchar *store_string;
-	gint key_len;
 	gboolean retval;
+	GtkTreePath *path;
 
 	gtk_tree_model_get (model, iter, column, &store_string, -1);
 
-	if (!store_string) return FALSE;
+	if (!store_string || !key) return FALSE;
 
-	key_len = strlen (key);
 
-	retval = (strncmp (key, store_string, key_len) != 0);
+	retval = (strncmp (key, store_string, strlen(key)) != 0);
 
 	g_free(store_string);
+	debug_print("selecting row\n");
+	path = gtk_tree_model_get_path(model, iter);
+	prefs_filtering_select_row(GTK_TREE_VIEW(filtering.cond_list_view), path);
+	gtk_tree_path_free(path);
 
 	return retval;
 }
@@ -1034,14 +1051,15 @@ static void prefs_filtering_set_list(void)
 static gint prefs_filtering_list_view_set_row(gint row, FilteringProp * prop)
 {
 	GtkTreeView *list_view = GTK_TREE_VIEW(filtering.cond_list_view);
-	gchar *str;
+	gchar *str = NULL;
 	GtkListStore *list_store;
 	gchar *name = NULL;
 	gint account_id = 0;
 	gchar *account_name = (gchar *)Q_("Filtering Account Menu|All");
 	gboolean enabled = TRUE;
 
-	str = filteringprop_to_string(prop);
+	if (prop)
+		str = filteringprop_to_string(prop);
 	if (str == NULL)
 		return -1;
 
@@ -1084,7 +1102,7 @@ static void prefs_filtering_condition_define_done(MatcherList * matchers)
 	}
 }
 
-static void prefs_filtering_condition_define(void)
+static void prefs_filtering_condition_define(gpointer action, gpointer data)
 {
 	gchar * cond_str;
 	MatcherList * matchers = NULL;
@@ -1120,7 +1138,7 @@ static void prefs_filtering_action_define_done(GSList * action_list)
 	}
 }
 
-static void prefs_filtering_action_define(void)
+static void prefs_filtering_action_define(gpointer action, gpointer data)
 {
 	gchar * action_str;
 	GSList * action_list = NULL;
@@ -1201,7 +1219,7 @@ fail:
 	return prop;
 }
 
-static void prefs_filtering_register_cb(void)
+static void prefs_filtering_register_cb(gpointer action, gpointer data)
 {
 	FilteringProp *prop;
 	
@@ -1216,7 +1234,7 @@ static void prefs_filtering_register_cb(void)
 	modified = TRUE;
 }
 
-static void prefs_filtering_substitute_cb(void)
+static void prefs_filtering_substitute_cb(gpointer action, gpointer data)
 {
 	gint selected_row = gtkut_list_view_get_selected_row
 		(filtering.cond_list_view);
@@ -1248,7 +1266,7 @@ static void prefs_filtering_substitute_cb(void)
 	modified = TRUE;
 }
 
-static void prefs_filtering_delete_cb(void)
+static void prefs_filtering_delete_cb(gpointer action, gpointer data)
 {
 	GtkTreeView *list_view = GTK_TREE_VIEW(filtering.cond_list_view);
 	GtkTreeModel *model;
@@ -1274,7 +1292,7 @@ static void prefs_filtering_delete_cb(void)
 	modified = TRUE;
 }
 
-static void prefs_filtering_delete_all_cb(void)
+static void prefs_filtering_delete_all_cb(gpointer action, gpointer data)
 {
 	GtkListStore *list_store;
 	
@@ -1290,12 +1308,12 @@ static void prefs_filtering_delete_all_cb(void)
 	modified = TRUE;
 }
 
-static void prefs_filtering_clear_cb(void)
+static void prefs_filtering_clear_cb(gpointer action, gpointer data)
 {
 	prefs_filtering_reset_dialog();
 }
 
-static void prefs_filtering_duplicate_cb(void)
+static void prefs_filtering_duplicate_cb(gpointer action, gpointer data)
 {
 	gint selected_row = gtkut_list_view_get_selected_row
 		(filtering.cond_list_view);	
@@ -1326,7 +1344,7 @@ static void prefs_filtering_duplicate_cb(void)
 	modified = TRUE;
 }
 
-static void prefs_filtering_top(void)
+static void prefs_filtering_top(gpointer action, gpointer data)
 {
 	gint row;
 	GtkTreeIter top, sel;
@@ -1347,7 +1365,7 @@ static void prefs_filtering_top(void)
 	modified = TRUE;
 }
 
-static void prefs_filtering_page_up(void)
+static void prefs_filtering_page_up(gpointer action, gpointer data)
 {
 	gint row, target_row, n_rows;
 	GtkTreeIter selected, target;
@@ -1379,7 +1397,7 @@ static void prefs_filtering_page_up(void)
 	gtkut_list_view_select_row(filtering.cond_list_view, target_row);
 	modified = TRUE;
 }
-static void prefs_filtering_up(void)
+static void prefs_filtering_up(gpointer action, gpointer data)
 {
 	gint row;
 	GtkTreeIter top, sel;
@@ -1400,7 +1418,7 @@ static void prefs_filtering_up(void)
 	modified = TRUE;
 }
 
-static void prefs_filtering_down(void)
+static void prefs_filtering_down(gpointer action, gpointer data)
 {
 	gint row, n_rows;
 	GtkTreeIter top, sel;
@@ -1421,7 +1439,7 @@ static void prefs_filtering_down(void)
 	modified = TRUE;
 }
 
-static void prefs_filtering_page_down(void)
+static void prefs_filtering_page_down(gpointer action, gpointer data)
 {
 	gint row, target_row, n_rows;
 	GtkTreeIter selected, target;
@@ -1454,7 +1472,7 @@ static void prefs_filtering_page_down(void)
 	modified = TRUE;
 }
 
-static void prefs_filtering_bottom(void)
+static void prefs_filtering_bottom(gpointer action, gpointer data)
 {
 	gint row, n_rows;
 	GtkTreeIter top, sel;
@@ -1507,7 +1525,7 @@ static void prefs_filtering_select_set(FilteringProp *prop)
 static gint prefs_filtering_deleted(GtkWidget *widget, GdkEventAny *event,
 				 gpointer data)
 {
-	prefs_filtering_cancel();
+	prefs_filtering_cancel(NULL, NULL);
 	return TRUE;
 }
 
@@ -1515,7 +1533,7 @@ static gboolean prefs_filtering_key_pressed(GtkWidget *widget, GdkEventKey *even
 				     gpointer data)
 {
 	if (event && event->keyval == GDK_Escape) {
-		prefs_filtering_cancel();
+		prefs_filtering_cancel(NULL, NULL);
 		return TRUE;			
 	}
 	return FALSE;
@@ -1593,7 +1611,7 @@ static gboolean prefs_filtering_check_mod(gboolean check_changed_list)
 	return FALSE;
 }
 
-static void prefs_filtering_ok(void)
+static void prefs_filtering_ok(gpointer action, gpointer data)
 {
 	if (prefs_filtering_check_mod(FALSE))
 		return;
@@ -1603,7 +1621,7 @@ static void prefs_filtering_ok(void)
 	prefs_filtering_close();
 }
 
-static void prefs_filtering_cancel(void)
+static void prefs_filtering_cancel(gpointer action, gpointer data)
 {
 	if (prefs_filtering_check_mod(TRUE))
 		return;
@@ -1744,18 +1762,19 @@ static void prefs_filtering_list_view_get_rule_info(GtkWidget *list, gint row,
 	}
 }
 
-static GtkItemFactory *prefs_filtering_popup_factory = NULL;
+static GtkActionGroup *prefs_filtering_popup_action = NULL;
 static GtkWidget *prefs_filtering_popup_menu = NULL;
 
-static GtkItemFactoryEntry prefs_filtering_popup_entries[] =
+static GtkActionEntry prefs_filtering_popup_entries[] =
 {
-	{N_("/_Delete"),		NULL, prefs_filtering_delete_cb, 0, NULL, NULL},
-	{N_("/Delete _all"),		NULL, prefs_filtering_delete_all_cb, 0, NULL, NULL},
-	{N_("/D_uplicate"),		NULL, prefs_filtering_duplicate_cb, 0, NULL, NULL},
-#ifdef MAEMO
-	{N_("/---"),			NULL, NULL, 0, "<Separator>", NULL},
-	{N_("/Move one page up"),		NULL, prefs_filtering_page_up, 0, NULL, NULL},
-	{N_("/Move one page down"),		NULL, prefs_filtering_page_down, 0, NULL, NULL},
+	{"PrefsFilteringPopup",			NULL, "PrefsFilteringPopup" },
+	{"PrefsFilteringPopup/Delete",		NULL, N_("_Delete"), NULL, NULL, G_CALLBACK(prefs_filtering_delete_cb) },
+	{"PrefsFilteringPopup/DeleteAll",	NULL, N_("Delete _all"), NULL, NULL, G_CALLBACK(prefs_filtering_delete_all_cb) },
+	{"PrefsFilteringPopup/Duplicate",	NULL, N_("D_uplicate"), NULL, NULL, G_CALLBACK(prefs_filtering_duplicate_cb) },
+#ifdef GENERIC_UMPC
+	{"PrefsFilteringPopup/---",		NULL, "---", NULL, NULL, NULL },
+	{"PrefsFilteringPopup/PageUp",		NULL, N_("Move one page up"), NULL, NULL, G_CALLBACK(prefs_filtering_page_up) },
+	{"PrefsFilteringPopup/PageDown",	NULL, N_("Move one page down"), NULL, NULL, G_CALLBACK(prefs_filtering_page_down) },
 #endif
 };
 
@@ -1783,24 +1802,32 @@ static gint prefs_filtering_list_btn_pressed(GtkWidget *widget, GdkEventButton *
 			gint row;
 
 			if (!prefs_filtering_popup_menu) {
-				gint n_entries = sizeof(prefs_filtering_popup_entries) /
-						sizeof(prefs_filtering_popup_entries[0]);
-				prefs_filtering_popup_menu = menu_create_items(prefs_filtering_popup_entries,
-								  n_entries, "<PrefsFilteringPopupMenu>",
-								  &prefs_filtering_popup_factory, list_view);
+				prefs_filtering_popup_action = cm_menu_create_action_group("PrefsFilteringPopup", prefs_filtering_popup_entries,
+					G_N_ELEMENTS(prefs_filtering_popup_entries), (gpointer)list_view);
+				MENUITEM_ADDUI("/Menus", "PrefsFilteringPopup", "PrefsFilteringPopup", GTK_UI_MANAGER_MENU)
+				MENUITEM_ADDUI("/Menus/PrefsFilteringPopup", "Delete", "PrefsFilteringPopup/Delete", GTK_UI_MANAGER_MENUITEM)
+				MENUITEM_ADDUI("/Menus/PrefsFilteringPopup", "DeleteAll", "PrefsFilteringPopup/DeleteAll", GTK_UI_MANAGER_MENUITEM)
+				MENUITEM_ADDUI("/Menus/PrefsFilteringPopup", "Duplicate", "PrefsFilteringPopup/Duplicate", GTK_UI_MANAGER_MENUITEM)
+#ifdef GENERIC_UMPC
+				MENUITEM_ADDUI("/Menus/PrefsFilteringPopup", "Separator1", "PrefsFilteringPopup/---", GTK_UI_MANAGER_SEPARATOR)
+				MENUITEM_ADDUI("/Menus/PrefsFilteringPopup", "PageUp", "PrefsFilteringPopup/PageUp", GTK_UI_MANAGER_MENUITEM)
+				MENUITEM_ADDUI("/Menus/PrefsFilteringPopup", "PageDown", "PrefsFilteringPopup/PageDown", GTK_UI_MANAGER_MENUITEM)
+#endif
+				prefs_filtering_popup_menu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(
+					gtk_ui_manager_get_widget(gtkut_ui_manager(), "/Menus/PrefsFilteringPopup")) );
 			}
 
 			/* grey out some popup menu items if there is no selected row */
 			row = gtkut_list_view_get_selected_row(GTK_WIDGET(list_view));
-			menu_set_sensitive(prefs_filtering_popup_factory, "/Delete", (row > 0));
-			menu_set_sensitive(prefs_filtering_popup_factory, "/Duplicate", (row > 0));
+			cm_menu_set_sensitive("PrefsFilteringPopup/Delete", (row > 0));
+			cm_menu_set_sensitive("PrefsFilteringPopup/Duplicate", (row > 0));
 
 			/* grey out seom popup menu items if there is no row
 			   (not counting the (New) one at row 0) */
 			non_empty = gtk_tree_model_get_iter_first(model, &iter);
 			if (non_empty)
 				non_empty = gtk_tree_model_iter_next(model, &iter);
-			menu_set_sensitive(prefs_filtering_popup_factory, "/Delete all", non_empty);
+			cm_menu_set_sensitive("PrefsFilteringPopup/DeleteAll", non_empty);
 
 			gtk_menu_popup(GTK_MENU(prefs_filtering_popup_menu), 
 			    	   NULL, NULL, NULL, NULL, 
@@ -1833,7 +1860,7 @@ static GtkWidget *prefs_filtering_list_view_create(void)
 
 	list_view = GTK_TREE_VIEW(gtk_tree_view_new_with_model(GTK_TREE_MODEL
 		(prefs_filtering_create_data_store())));
-#ifdef MAEMO
+#ifdef GENERIC_UMPC
 	g_object_set(list_view, "allow-checkbox-mode", FALSE, NULL);
 #endif
 
@@ -1929,7 +1956,7 @@ static void prefs_filtering_create_list_view_columns(GtkWidget *list_view)
 		 "text", PREFS_FILTERING_RULE,
 		 NULL);
 
-	gtk_tree_view_set_search_column(GTK_TREE_VIEW(list_view), 1);
+	gtk_tree_view_set_search_column(GTK_TREE_VIEW(list_view), PREFS_FILTERING_NAME);
 	gtk_tree_view_set_search_equal_func(GTK_TREE_VIEW(list_view), prefs_filtering_search_func_cb , NULL, NULL);
 	
 	gtk_tree_view_append_column(GTK_TREE_VIEW(list_view), column);		
