@@ -1,10 +1,10 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2005-2007 Colin Leroy <colin@colino.net> & The Claws Mail Team
+ * Copyright (C) 2005-2011 Colin Leroy <colin@colino.net> & The Claws Mail Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -31,6 +31,10 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
+#ifdef G_OS_WIN32
+#  include <w32lib.h>
+#endif
+
 #include "prefs_common.h"
 #include "prefs_gtk.h"
 #include "prefs_summary_open.h"
@@ -40,6 +44,7 @@
 #include "gtk/menu.h"
 #include "gtk/gtkutils.h"
 #include "gtk/prefswindow.h"
+#include "gtk/combobox.h"
 
 #include "manage_window.h"
 
@@ -53,12 +58,13 @@ typedef struct _SummariesPage
 	GtkWidget *optmenu_folder_unread;
 	GtkWidget *spinbtn_ng_abbrev_len;
 	GtkWidget *checkbtn_useaddrbook;
+	GtkWidget *checkbtn_show_tooltips;
 	GtkWidget *checkbtn_threadsubj;
 	GtkWidget *button_datefmt;
 	GtkWidget *entry_datefmt;
 
 	GtkWidget *checkbtn_reopen_last_folder;
-	GtkWidget *checkbtn_always_show_msg;
+	GtkWidget *optmenu_always_show_msg;
 	GtkWidget *checkbtn_mark_as_read_on_newwin;
 	GtkWidget *spinbtn_mark_as_read_delay;
 	GtkWidget *checkbtn_immedexec;
@@ -89,6 +95,8 @@ static void date_format_select_row	        (GtkTreeView *list_view,
 						 GtkTreePath *path,
 						 GtkTreeViewColumn *column,
 						 GtkWidget *date_format);
+static void mark_as_read_toggled		(GtkToggleButton *button,
+						 GtkWidget *spinbtn);
 
 static GtkWidget *date_format_create(GtkButton *button, void *data)
 {
@@ -140,7 +148,7 @@ static GtkWidget *date_format_create(GtkButton *button, void *data)
 	GtkTreeViewColumn *column;
 	GtkTreeSelection *selection;
 
-	time_format[0].txt  = _("the full abbreviated weekday name");
+	time_format[0].txt  = _("the abbreviated weekday name");
 	time_format[1].txt  = _("the full weekday name");
 	time_format[2].txt  = _("the abbreviated month name");
 	time_format[3].txt  = _("the full month name");
@@ -213,7 +221,7 @@ static GtkWidget *date_format_create(GtkButton *button, void *data)
 			 NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(datefmt_list_view), column);
 	
-	/* gtk_clist_set_column_width(GTK_CLIST(datefmt_clist), 0, 80); */
+	/* gtk_cmclist_set_column_width(GTK_CMCLIST(datefmt_clist), 0, 80); */
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(datefmt_list_view));
 	gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
 
@@ -312,35 +320,32 @@ static void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 	GtkWidget *spinbtn_ng_abbrev_len;
 	GtkObject *spinbtn_ng_abbrev_len_adj;
 	GtkWidget *vbox2;
+	GtkWidget *vbox3;
 	GtkWidget *checkbtn_useaddrbook;
+	GtkWidget *checkbtn_show_tooltips;
 	GtkWidget *checkbtn_threadsubj;
 	GtkWidget *label_datefmt;
 	GtkWidget *button_datefmt;
 	GtkWidget *entry_datefmt;
-	GtkTooltips *tip_datefmt;
 	GtkWidget *hbox_dispitem;
 	GtkWidget *button_dispitem;
-
 	GtkWidget *hbox2;
 	GtkWidget *checkbtn_reopen_last_folder;
-	GtkWidget *checkbtn_always_show_msg;
-	GtkWidget *checkbtn_mark_as_read_on_newwin;
+	GtkWidget *optmenu_always_show_msg;
 	GtkWidget *spinbtn_mark_as_read_delay;
 	GtkObject *spinbtn_mark_as_read_delay_adj;
 	GtkWidget *checkbtn_immedexec;
 	GtkWidget *checkbtn_ask_mark_all_read;
-	GtkTooltips *immedexec_tooltip;
 	GtkWidget *label, *label_fill;
-	GtkWidget *menu;
-	GtkWidget *menuitem;
+	GtkListStore *menu;
+	GtkTreeIter iter;
  	GtkWidget *optmenu_nextunreadmsgdialog;
 	GtkWidget *folderview_frame;
 	GtkWidget *summaryview_frame;
 	GtkWidget *button_edit_actions;
-
-	GtkTooltips *tooltips;
-
-	tooltips = gtk_tooltips_new();
+	GtkWidget *radio_mark_as_read_on_select;
+	GtkWidget *radio_mark_as_read_on_new_win;
+	CLAWS_TIP_DECL();
 
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
 	gtk_widget_show (vbox1);
@@ -356,15 +361,15 @@ static void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 	gtk_widget_show (label);
 	gtk_box_pack_start(GTK_BOX(hbox0), label, FALSE, FALSE, 0);
 
- 	optmenu_folder_unread = gtk_option_menu_new ();
- 	gtk_widget_show (optmenu_folder_unread);
+	optmenu_folder_unread = gtkut_sc_combobox_create(NULL, FALSE);
+	menu = GTK_LIST_STORE(gtk_combo_box_get_model(
+				GTK_COMBO_BOX(optmenu_folder_unread)));
+	gtk_widget_show (optmenu_folder_unread);
  	
-	menu = gtk_menu_new ();
-	MENUITEM_ADD (menu, menuitem, _("No"), 0);
-	MENUITEM_ADD (menu, menuitem, _("Unread messages"), 1);
-	MENUITEM_ADD (menu, menuitem, _("Unread and Total messages"), 2);
+	COMBOBOX_ADD (menu, _("No"), 0);
+	COMBOBOX_ADD (menu, _("Unread messages"), 1);
+	COMBOBOX_ADD (menu, _("Unread and Total messages"), 2);
 
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (optmenu_folder_unread), menu);
 	gtk_box_pack_start(GTK_BOX(hbox0), optmenu_folder_unread, FALSE, FALSE, 0);
 
 	hbox1 = gtk_hbox_new (FALSE, 8);
@@ -373,14 +378,14 @@ static void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 
 	PACK_CHECK_BUTTON
 		(vbox2, checkbtn_reopen_last_folder,
-		 _("Open last opened folder at startup"));
+		 _("Open last opened folder at start-up"));
 
 	label_ng_abbrev = gtk_label_new
 		(_("Abbreviate newsgroup names longer than"));
 	gtk_widget_show (label_ng_abbrev);
 	gtk_box_pack_start (GTK_BOX (hbox1), label_ng_abbrev, FALSE, FALSE, 0);
 
-	spinbtn_ng_abbrev_len_adj = gtk_adjustment_new (16, 0, 999, 1, 10, 10);
+	spinbtn_ng_abbrev_len_adj = gtk_adjustment_new (16, 0, 999, 1, 10, 0);
 	spinbtn_ng_abbrev_len = gtk_spin_button_new
 		(GTK_ADJUSTMENT (spinbtn_ng_abbrev_len_adj), 1, 0);
 	gtk_widget_show (spinbtn_ng_abbrev_len);
@@ -431,63 +436,85 @@ static void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 	gtk_widget_show (label);
 	gtk_box_pack_start(GTK_BOX(hbox1), label, FALSE, FALSE, 0);
 	
- 	optmenu_nextunreadmsgdialog = gtk_option_menu_new ();
- 	gtk_widget_show (optmenu_nextunreadmsgdialog);
+	optmenu_nextunreadmsgdialog = gtkut_sc_combobox_create(NULL, FALSE);
+	menu = GTK_LIST_STORE(gtk_combo_box_get_model(
+				GTK_COMBO_BOX(optmenu_nextunreadmsgdialog)));
+	gtk_widget_show (optmenu_nextunreadmsgdialog);
 
-	menu = gtk_menu_new ();
-	MENUITEM_ADD (menu, menuitem, _("Always"), NEXTUNREADMSGDIALOG_ALWAYS);
-	MENUITEM_ADD (menu, menuitem, _("Assume 'Yes'"), 
-		      NEXTUNREADMSGDIALOG_ASSUME_YES);
-	MENUITEM_ADD (menu, menuitem, _("Assume 'No'"), 
-		      NEXTUNREADMSGDIALOG_ASSUME_NO);
+	COMBOBOX_ADD (menu, _("Always"), NEXTUNREADMSGDIALOG_ALWAYS);
+	COMBOBOX_ADD (menu, _("Assume 'Yes'"), NEXTUNREADMSGDIALOG_ASSUME_YES);
+	COMBOBOX_ADD (menu, _("Assume 'No'"), NEXTUNREADMSGDIALOG_ASSUME_NO);
 
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (optmenu_nextunreadmsgdialog), menu);
 	gtk_box_pack_start(GTK_BOX(hbox1), optmenu_nextunreadmsgdialog, FALSE, FALSE, 0);
 
-	PACK_CHECK_BUTTON
-		(vbox2, checkbtn_always_show_msg,
-		 _("Always open message when selected"));
+	/* Open message on select policy */
+	hbox1 = gtk_hbox_new (FALSE, 10);
+	gtk_widget_show (hbox1);
+	gtk_box_pack_start (GTK_BOX (vbox2), hbox1, FALSE, FALSE, 0);	
+	label = gtk_label_new (_("Open message when selected"));
+	gtk_widget_show (label);
+	gtk_box_pack_start(GTK_BOX(hbox1), label, FALSE, FALSE, 0);
+
+	optmenu_always_show_msg = gtkut_sc_combobox_create(NULL, FALSE);
+	menu = GTK_LIST_STORE(gtk_combo_box_get_model(
+				GTK_COMBO_BOX(optmenu_always_show_msg)));
+	gtk_widget_show (optmenu_always_show_msg);
+	COMBOBOX_ADD (menu, _("Never"), OPENMSG_REQUEST_ONLY);
+	COMBOBOX_ADD (menu, _("Always"), OPENMSG_ALWAYS);
+	COMBOBOX_ADD (menu, _("When message view is visible"),
+			OPENMSG_WHEN_VIEW_VISIBLE);
+	gtk_box_pack_start(GTK_BOX(hbox1), optmenu_always_show_msg, FALSE, FALSE, 0);
+
 	PACK_CHECK_BUTTON
 		(vbox2, checkbtn_threadsubj,
 		 _("Thread using subject in addition to standard headers"));
 
-	immedexec_tooltip = gtk_tooltips_new();
-
 	PACK_CHECK_BUTTON
 		(vbox2, checkbtn_immedexec,
 		 _("Execute immediately when moving or deleting messages"));
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(immedexec_tooltip), checkbtn_immedexec,
+	CLAWS_SET_TIP(checkbtn_immedexec,
 			     _("Defers moving, copying and deleting of messages"
-		   	       " until you choose 'Tools/Execute'"),
-			     NULL);
-	PACK_CHECK_BUTTON
-		(vbox2, checkbtn_mark_as_read_on_newwin,
-		 _("Only mark message as read when opened \n"
-		   "in a new window, or replied to"));
-		 
+		   	       " until you choose 'Tools/Execute'"));
+
+	vbox3 = gtkut_get_options_frame(vbox2, NULL, _("Mark message as read"));
+
+	radio_mark_as_read_on_select = gtk_radio_button_new_with_label(NULL,
+			_("when selected, after"));
+
 	hbox1 = gtk_hbox_new (FALSE, 8);
-	gtk_widget_show (hbox1);
-	gtk_box_pack_start (GTK_BOX (vbox2), hbox1, FALSE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox1), radio_mark_as_read_on_select, FALSE, FALSE, 0);
 
-	gtk_box_pack_start (GTK_BOX (hbox1), gtk_label_new
-		(_("Mark messages as read after")), FALSE, FALSE, 0);
-
-	spinbtn_mark_as_read_delay_adj = gtk_adjustment_new (0, 0, 60, 1, 10, 10);
+	spinbtn_mark_as_read_delay_adj = gtk_adjustment_new (0, 0, 60, 1, 10, 0);
 	spinbtn_mark_as_read_delay = gtk_spin_button_new
-		(GTK_ADJUSTMENT (spinbtn_mark_as_read_delay_adj), 1, 0);
+			(GTK_ADJUSTMENT (spinbtn_mark_as_read_delay_adj), 1, 0);
 	gtk_box_pack_start (GTK_BOX (hbox1), spinbtn_mark_as_read_delay,
 			    FALSE, FALSE, 0);
 	gtk_widget_set_size_request (spinbtn_mark_as_read_delay, 56, -1);
 	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbtn_mark_as_read_delay),
 				     TRUE);
 	gtk_box_pack_start (GTK_BOX (hbox1), gtk_label_new
-		(_("seconds")), FALSE, FALSE, 0);
-	gtk_widget_show_all(hbox1);
+			(_("seconds")), FALSE, FALSE, 0);
 
+	gtk_box_pack_start (GTK_BOX (vbox3), hbox1, FALSE, FALSE, 0);
+
+	radio_mark_as_read_on_new_win = gtk_radio_button_new_with_label_from_widget(
+			GTK_RADIO_BUTTON(radio_mark_as_read_on_select),
+			_("only when opened in a new window, or replied to"));
+	gtk_box_pack_start (GTK_BOX (vbox3), radio_mark_as_read_on_new_win,
+			FALSE, FALSE, 0);
+	gtk_widget_show_all(vbox3);
+                                                                                             
 	PACK_CHECK_BUTTON
 		(vbox2, checkbtn_useaddrbook,
 		 _("Display sender using address book"));
 		 
+	PACK_CHECK_BUTTON
+		(vbox2, checkbtn_show_tooltips,
+		 _("Show tooltips"));
+
+#if !GTK_CHECK_VERSION(2,12,0) && !GENERIC_UMPC
+	gtk_widget_hide(checkbtn_show_tooltips);
+#endif
 	hbox2 = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox2);
 	gtk_box_pack_start (GTK_BOX (vbox2), hbox2, FALSE, TRUE, 0);
@@ -501,11 +528,7 @@ static void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 	gtk_widget_set_size_request(entry_datefmt, 200, -1);
 	gtk_box_pack_start (GTK_BOX (hbox2), entry_datefmt, FALSE, FALSE, 0);
 
-#if GTK_CHECK_VERSION(2, 8, 0)
 	button_datefmt = gtk_button_new_from_stock(GTK_STOCK_INFO);
-#else
-	button_datefmt = gtk_button_new_with_label ("Info...");
-#endif
 
 	gtk_widget_show (button_datefmt);
 	gtk_box_pack_start (GTK_BOX (hbox2), button_datefmt, FALSE, FALSE, 0);
@@ -515,10 +538,8 @@ static void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 	label_fill = gtk_label_new(" ");
 	gtk_box_pack_start(GTK_BOX(hbox2), label_fill, TRUE, FALSE, 0);
 	
-	tip_datefmt = gtk_tooltips_new();
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tip_datefmt),
-			     button_datefmt,
-			     _("Date format help"), NULL);
+	CLAWS_SET_TIP(button_datefmt,
+			     _("Date format help"));
 			     
 	hbox_dispitem = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox_dispitem);
@@ -536,27 +557,32 @@ static void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 
 	PACK_CHECK_BUTTON
 		(vbox1, checkbtn_ask_mark_all_read,
-		 _("Confirm before marking all mails in a folder as read"));
+		 _("Confirm before marking all messages in a folder as read"));
 	PACK_CHECK_BUTTON
 		(vbox1, checkbtn_transhdr,
 		 _("Translate header names"));
-	gtk_tooltips_set_tip(tooltips, checkbtn_transhdr,
+	CLAWS_SET_TIP(checkbtn_transhdr,
 			     _("The display of standard headers (such as 'From:', 'Subject:') "
-			     "will be translated into your language."), NULL);
+			     "will be translated into your language."));
 	
 	hbox2 = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox2);
 	gtk_box_pack_start (GTK_BOX (vbox1), hbox2, FALSE, FALSE, 0);
 
+	g_signal_connect(G_OBJECT(radio_mark_as_read_on_select), "toggled",
+			 G_CALLBACK(mark_as_read_toggled),
+			 spinbtn_mark_as_read_delay);
 
 	prefs_summaries->window			= GTK_WIDGET(window);
 	
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_transhdr),
 			prefs_common.trans_hdr);
-	gtk_option_menu_set_history(GTK_OPTION_MENU(optmenu_folder_unread),
+	combobox_select_by_data(GTK_COMBO_BOX(optmenu_folder_unread),
 			prefs_common.display_folder_unread);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_useaddrbook),
 			prefs_common.use_addr_book);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_show_tooltips),
+			prefs_common.show_tooltips);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_threadsubj),
 			prefs_common.thread_by_subject);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbtn_ng_abbrev_len),
@@ -566,9 +592,9 @@ static void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_reopen_last_folder),
 			prefs_common.goto_last_folder_on_startup);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_always_show_msg),
+	combobox_select_by_data(GTK_COMBO_BOX(optmenu_always_show_msg),
 			prefs_common.always_show_msg);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_mark_as_read_on_newwin),
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_mark_as_read_on_new_win),
 			prefs_common.mark_as_read_on_new_window);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbtn_mark_as_read_delay),
 			prefs_common.mark_as_read_delay);
@@ -577,19 +603,20 @@ static void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_ask_mark_all_read),
 			prefs_common.ask_mark_all_read);
 
-	gtk_option_menu_set_history(GTK_OPTION_MENU(optmenu_nextunreadmsgdialog),
+	combobox_select_by_data(GTK_COMBO_BOX(optmenu_nextunreadmsgdialog),
 			prefs_common.next_unread_msg_dialog);
 
 	prefs_summaries->checkbtn_transhdr = checkbtn_transhdr;
 	prefs_summaries->optmenu_folder_unread = optmenu_folder_unread;
 	prefs_summaries->spinbtn_ng_abbrev_len = spinbtn_ng_abbrev_len;
 	prefs_summaries->checkbtn_useaddrbook = checkbtn_useaddrbook;
+	prefs_summaries->checkbtn_show_tooltips = checkbtn_show_tooltips;
 	prefs_summaries->checkbtn_threadsubj = checkbtn_threadsubj;
 	prefs_summaries->entry_datefmt = entry_datefmt;
 
 	prefs_summaries->checkbtn_reopen_last_folder = checkbtn_reopen_last_folder;
-	prefs_summaries->checkbtn_always_show_msg = checkbtn_always_show_msg;
-	prefs_summaries->checkbtn_mark_as_read_on_newwin = checkbtn_mark_as_read_on_newwin;
+	prefs_summaries->optmenu_always_show_msg = optmenu_always_show_msg;
+	prefs_summaries->checkbtn_mark_as_read_on_newwin = radio_mark_as_read_on_new_win;
 	prefs_summaries->spinbtn_mark_as_read_delay = spinbtn_mark_as_read_delay;
 	prefs_summaries->checkbtn_immedexec = checkbtn_immedexec;
 	prefs_summaries->checkbtn_ask_mark_all_read = checkbtn_ask_mark_all_read;
@@ -601,19 +628,18 @@ static void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 static void prefs_summaries_save(PrefsPage *_page)
 {
 	SummariesPage *page = (SummariesPage *) _page;
-	GtkWidget *menu;
-	GtkWidget *menuitem;
 
 	prefs_common.trans_hdr = gtk_toggle_button_get_active(
 			GTK_TOGGLE_BUTTON(page->checkbtn_transhdr));
 
-	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(page->optmenu_folder_unread));
-	menuitem = gtk_menu_get_active(GTK_MENU(menu));
-	prefs_common.display_folder_unread = GPOINTER_TO_INT
-		(g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID));
+
+	prefs_common.display_folder_unread = combobox_get_active_data(
+			GTK_COMBO_BOX(page->optmenu_folder_unread));
 
 	prefs_common.use_addr_book = gtk_toggle_button_get_active(
 			GTK_TOGGLE_BUTTON(page->checkbtn_useaddrbook));
+	prefs_common.show_tooltips = gtk_toggle_button_get_active(
+			GTK_TOGGLE_BUTTON(page->checkbtn_show_tooltips));
 	prefs_common.thread_by_subject = gtk_toggle_button_get_active(
 			GTK_TOGGLE_BUTTON(page->checkbtn_threadsubj));
 	prefs_common.ng_abbrev_len = gtk_spin_button_get_value_as_int(
@@ -625,8 +651,8 @@ static void prefs_summaries_save(PrefsPage *_page)
 
 	prefs_common.goto_last_folder_on_startup = gtk_toggle_button_get_active(
 		GTK_TOGGLE_BUTTON(page->checkbtn_reopen_last_folder));
-	prefs_common.always_show_msg = gtk_toggle_button_get_active(
-		GTK_TOGGLE_BUTTON(page->checkbtn_always_show_msg));
+	prefs_common.always_show_msg = combobox_get_active_data(
+		GTK_COMBO_BOX(page->optmenu_always_show_msg));
 	prefs_common.mark_as_read_on_new_window = gtk_toggle_button_get_active(
 		GTK_TOGGLE_BUTTON(page->checkbtn_mark_as_read_on_newwin));
 	prefs_common.immediate_exec = gtk_toggle_button_get_active(
@@ -636,10 +662,8 @@ static void prefs_summaries_save(PrefsPage *_page)
 	prefs_common.mark_as_read_delay = gtk_spin_button_get_value_as_int(
 			GTK_SPIN_BUTTON(page->spinbtn_mark_as_read_delay));
 
-	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(page->optmenu_nextunreadmsgdialog));
-	menuitem = gtk_menu_get_active(GTK_MENU(menu));
-	prefs_common.next_unread_msg_dialog = GPOINTER_TO_INT
-		(g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID));
+	prefs_common.next_unread_msg_dialog = combobox_get_active_data(
+			GTK_COMBO_BOX(page->optmenu_nextunreadmsgdialog));
 	main_window_reflect_prefs_all();
 }
 
@@ -679,13 +703,13 @@ static void date_format_ok_btn_clicked(GtkButton *button, GtkWidget **widget)
 	GtkWidget *datefmt_sample = NULL;
 	gchar *text;
 
-	g_return_if_fail(widget != NULL);
-	g_return_if_fail(*widget != NULL);
-	g_return_if_fail(prefs_summaries->entry_datefmt != NULL);
+	cm_return_if_fail(widget != NULL);
+	cm_return_if_fail(*widget != NULL);
+	cm_return_if_fail(prefs_summaries->entry_datefmt != NULL);
 
 	datefmt_sample = GTK_WIDGET(g_object_get_data
 				    (G_OBJECT(*widget), "datefmt_sample"));
-	g_return_if_fail(datefmt_sample != NULL);
+	cm_return_if_fail(datefmt_sample != NULL);
 
 	text = gtk_editable_get_chars(GTK_EDITABLE(datefmt_sample), 0, -1);
 	g_free(prefs_common.date_format);
@@ -699,8 +723,8 @@ static void date_format_ok_btn_clicked(GtkButton *button, GtkWidget **widget)
 static void date_format_cancel_btn_clicked(GtkButton *button,
 					   GtkWidget **widget)
 {
-	g_return_if_fail(widget != NULL);
-	g_return_if_fail(*widget != NULL);
+	cm_return_if_fail(widget != NULL);
+	cm_return_if_fail(*widget != NULL);
 
 	gtk_widget_destroy(*widget);
 	*widget = NULL;
@@ -717,8 +741,8 @@ static gboolean date_format_key_pressed(GtkWidget *keywidget, GdkEventKey *event
 static gboolean date_format_on_delete(GtkWidget *dialogwidget,
 				      GdkEventAny *event, GtkWidget **widget)
 {
-	g_return_val_if_fail(widget != NULL, FALSE);
-	g_return_val_if_fail(*widget != NULL, FALSE);
+	cm_return_val_if_fail(widget != NULL, FALSE);
+	cm_return_val_if_fail(*widget != NULL, FALSE);
 
 	*widget = NULL;
 	return FALSE;
@@ -739,15 +763,8 @@ static void date_format_entry_on_change(GtkEditable *editable,
 	text = gtk_editable_get_chars(editable, 0, -1);
 	if (text)
 		fast_strftime(buffer, sizeof buffer, text, cal_time); 
-	g_free(text);
 
-	text = conv_codeset_strdup(buffer,
-				   conv_get_locale_charset_str(),
-				   CS_UTF_8);
-	if (!text)
-		text = g_strdup(buffer);
-
-	gtk_label_set_text(example, text);
+	gtk_label_set_text(example, buffer);
 
 	g_free(text);
 }
@@ -765,13 +782,13 @@ static void date_format_select_row(GtkTreeView *list_view,
 	GtkTreeIter iter;
 	GtkTreeModel *model;
 	
-	g_return_if_fail(date_format != NULL);
+	cm_return_if_fail(date_format != NULL);
 
 	/* only on double click */
 	datefmt_sample = GTK_WIDGET(g_object_get_data(G_OBJECT(date_format), 
 						      "datefmt_sample"));
 
-	g_return_if_fail(datefmt_sample != NULL);
+	cm_return_if_fail(datefmt_sample != NULL);
 
 	model = gtk_tree_view_get_model(list_view);
 
@@ -797,3 +814,10 @@ static void date_format_select_row(GtkTreeView *list_view,
 
 	g_free(new_format);
 }
+
+static void mark_as_read_toggled(GtkToggleButton *button, GtkWidget *spinbtn)
+{
+       gtk_widget_set_sensitive(spinbtn,
+               gtk_toggle_button_get_active(button));
+}
+ 
