@@ -1,10 +1,10 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 1999-2011 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -25,9 +25,7 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtkwindow.h>
+#include <gtk/gtk.h>
 #include <stdio.h>
 #include <string.h>
 #include <signal.h>
@@ -89,9 +87,9 @@ gint send_message(const gchar *file, PrefsAccount *ac_prefs, GSList *to_list)
 	FILE *fp;
 	gint val;
 
-	g_return_val_if_fail(file != NULL, -1);
-	g_return_val_if_fail(ac_prefs != NULL, -1);
-	g_return_val_if_fail(to_list != NULL, -1);
+	cm_return_val_if_fail(file != NULL, -1);
+	cm_return_val_if_fail(ac_prefs != NULL, -1);
+	cm_return_val_if_fail(to_list != NULL, -1);
 
 	if ((fp = g_fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
@@ -131,8 +129,8 @@ gint send_message_local(const gchar *command, FILE *fp)
 	gboolean err = FALSE;
 	gint status;
 
-	g_return_val_if_fail(command != NULL, -1);
-	g_return_val_if_fail(fp != NULL, -1);
+	cm_return_val_if_fail(command != NULL, -1);
+	cm_return_val_if_fail(fp != NULL, -1);
 
 	log_message(LOG_PROTOCOL, _("Sending message using command: %s\n"), command);
 
@@ -207,11 +205,11 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 	long fp_pos = 0;
 	gchar spec_from[BUFFSIZE];
 
-	g_return_val_if_fail(ac_prefs != NULL, -1);
-	g_return_val_if_fail(ac_prefs->address != NULL, -1);
-	g_return_val_if_fail(ac_prefs->smtp_server != NULL, -1);
-	g_return_val_if_fail(to_list != NULL, -1);
-	g_return_val_if_fail(fp != NULL, -1);
+	cm_return_val_if_fail(ac_prefs != NULL, -1);
+	cm_return_val_if_fail(ac_prefs->address != NULL, -1);
+	cm_return_val_if_fail(ac_prefs->smtp_server != NULL, -1);
+	cm_return_val_if_fail(to_list != NULL, -1);
+	cm_return_val_if_fail(fp != NULL, -1);
 
 	/* get the From address used, not necessarily the ac_prefs',
 	 * because it's editable. */
@@ -232,7 +230,7 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 
 	if (!ac_prefs->session) {
 		/* we can't reuse a previously initialised session */
-		session = smtp_session_new();
+		session = smtp_session_new(ac_prefs);
 		smtp_session = SMTP_SESSION(session);
 
 		if (ac_prefs->set_domain && ac_prefs->domain && strlen(ac_prefs->domain)) {
@@ -248,39 +246,31 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 				if (ac_prefs->smtp_passwd)
 					smtp_session->pass =
 						g_strdup(ac_prefs->smtp_passwd);
-				else if (ac_prefs->tmp_smtp_pass)
-					smtp_session->pass =
-						g_strdup(ac_prefs->tmp_smtp_pass);
 				else {
 					smtp_session->pass =
-						input_dialog_query_password
+						input_dialog_query_password_keep
 							(ac_prefs->smtp_server,
-							 smtp_session->user);
+							 smtp_session->user,
+							 &(ac_prefs->session_smtp_passwd));
 					if (!smtp_session->pass) {
 						session_destroy(session);
 						return -1;
 					}
-					ac_prefs->tmp_smtp_pass =
-						g_strdup(smtp_session->pass);
 				}
 			} else {
 				smtp_session->user = g_strdup(ac_prefs->userid);
 				if (ac_prefs->passwd)
 					smtp_session->pass = g_strdup(ac_prefs->passwd);
-				else if (ac_prefs->tmp_pass)
-					smtp_session->pass =
-						g_strdup(ac_prefs->tmp_pass);
 				else {
 					smtp_session->pass =
-						input_dialog_query_password
+						input_dialog_query_password_keep
 							(ac_prefs->smtp_server,
-							 smtp_session->user);
+							 smtp_session->user,
+							 &(ac_prefs->session_smtp_passwd));
 					if (!smtp_session->pass) {
 						session_destroy(session);
 						return -1;
 					}
-					ac_prefs->tmp_pass =
-						g_strdup(smtp_session->pass);
 				}
 			}
 		} else {
@@ -288,13 +278,13 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 			smtp_session->pass = NULL;
 		}
 
-	#if USE_OPENSSL
+#ifdef USE_GNUTLS
 		port = ac_prefs->set_smtpport ? ac_prefs->smtpport :
 			ac_prefs->ssl_smtp == SSL_TUNNEL ? SSMTP_PORT : SMTP_PORT;
 		session->ssl_type = ac_prefs->ssl_smtp;
 		if (ac_prefs->ssl_smtp != SSL_NONE)
 			session->nonblocking = ac_prefs->use_nonblocking_ssl;
-	#else
+#else
 		if (ac_prefs->ssl_smtp != SSL_NONE) {
 			if (alertpanel_full(_("Insecure connection"),
 				_("This connection is configured to be secured "
@@ -311,7 +301,7 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 			}
 		}
 		port = ac_prefs->set_smtpport ? ac_prefs->smtpport : SMTP_PORT;
-	#endif
+#endif
 
 		dialog = send_progress_dialog_create();
 		dialog->session = session;
@@ -325,15 +315,15 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 		    && (ac_prefs->protocol == A_APOP || ac_prefs->protocol == A_POP3)
 		    && (time(NULL) - ac_prefs->last_pop_login_time) > (60 * ac_prefs->pop_before_smtp_timeout)) {
 			g_snprintf(buf, sizeof(buf), _("Doing POP before SMTP..."));
-			log_message(LOG_PROTOCOL, buf);
+			log_message(LOG_PROTOCOL, "%s\n", buf);
 			progress_dialog_set_label(dialog->dialog, buf);
 			progress_dialog_list_set_status(dialog->dialog, 0, _("POP before SMTP"));
 			GTK_EVENTS_FLUSH();
 			inc_pop_before_smtp(ac_prefs);
 		}
 
-		g_snprintf(buf, sizeof(buf), _("Connecting to SMTP server: %s ..."),
-			   ac_prefs->smtp_server);
+		g_snprintf(buf, sizeof(buf), _("Account '%s': Connecting to SMTP server: %s ..."),
+				ac_prefs->account_name, ac_prefs->smtp_server);
 		progress_dialog_set_label(dialog->dialog, buf);
 		log_message(LOG_PROTOCOL, "%s\n", buf);
 
@@ -378,17 +368,14 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 		smtp_from(smtp_session);
 	}
 
-	while (session_is_connected(session) && dialog->cancelled == FALSE
+	while (session_is_running(session) && dialog->cancelled == FALSE
 		&& SMTP_SESSION(session)->state != SMTP_MAIL_SENT_OK)
 		gtk_main_iteration();
 
 	if (SMTP_SESSION(session)->error_val == SM_AUTHFAIL) {
-		if (ac_prefs->smtp_userid && ac_prefs->tmp_smtp_pass) {
-			g_free(ac_prefs->tmp_smtp_pass);
-			ac_prefs->tmp_smtp_pass = NULL;
-		} else if (ac_prefs->tmp_pass) {
-			g_free(ac_prefs->tmp_pass);
-			ac_prefs->tmp_pass = NULL;
+		if (ac_prefs->session_smtp_passwd) {
+			g_free(ac_prefs->session_smtp_passwd);
+			ac_prefs->session_smtp_passwd = NULL;
 		}
 		ret = -1;
 	} else if (SMTP_SESSION(session)->state == SMTP_MAIL_SENT_OK) {
@@ -435,7 +422,7 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 		ac_prefs->session = SMTP_SESSION(session);
 
 
-	statusbar_pop_all();
+	statuswindow_pop_all();
 	statusbar_verbosity_set(FALSE);
 	return ret;
 }
@@ -452,7 +439,7 @@ static gint send_recv_message(Session *session, const gchar *msg, gpointer data)
 	SendProgressDialog *dialog = (SendProgressDialog *)data;
 	gchar *state_str = NULL;
 
-	g_return_val_if_fail(dialog != NULL, -1);
+	cm_return_val_if_fail(dialog != NULL, -1);
 
 	switch (smtp_session->state) {
 	case SMTP_READY:
@@ -461,12 +448,12 @@ static gint send_recv_message(Session *session, const gchar *msg, gpointer data)
 	case SMTP_HELO:
 		g_snprintf(buf, sizeof(buf), _("Sending HELO..."));
 		state_str = _("Authenticating");
-		statusbar_print_all(_("Sending message..."));
+		statuswindow_print_all(_("Sending message..."));
 		break;
 	case SMTP_EHLO:
 		g_snprintf(buf, sizeof(buf), _("Sending EHLO..."));
 		state_str = _("Authenticating");
-		statusbar_print_all(_("Sending message..."));
+		statuswindow_print_all(_("Sending message..."));
 		break;
 	case SMTP_AUTH:
 		g_snprintf(buf, sizeof(buf), _("Authenticating..."));
@@ -507,8 +494,9 @@ static gint send_send_data_progressive(Session *session, guint cur_len,
 {
 	gchar buf[BUFFSIZE];
 	SendProgressDialog *dialog = (SendProgressDialog *)data;
-
-	g_return_val_if_fail(dialog != NULL, -1);
+	MainWindow *mainwin = mainwindow_get_mainwindow();
+	
+	cm_return_val_if_fail(dialog != NULL, -1);
 
 	if (SMTP_SESSION(session)->state != SMTP_SEND_DATA &&
 	    SMTP_SESSION(session)->state != SMTP_EOM)
@@ -520,23 +508,38 @@ static gint send_send_data_progressive(Session *session, guint cur_len,
 	progress_dialog_set_fraction
 		(dialog->dialog, (total_len == 0) ? 0 : (gfloat)cur_len / (gfloat)total_len);
 
+	if (mainwin) {
+		if (!gtkut_widget_get_visible(mainwin->progressbar))	
+			gtk_widget_show(mainwin->progressbar);
+		gtk_progress_bar_set_fraction
+			(GTK_PROGRESS_BAR(mainwin->progressbar),
+			 (total_len == 0) ? 0 : (gfloat)cur_len / (gfloat)total_len);
+	}
+
 	return 0;
 }
 
 static gint send_send_data_finished(Session *session, guint len, gpointer data)
 {
 	SendProgressDialog *dialog = (SendProgressDialog *)data;
+	MainWindow *mainwin = mainwindow_get_mainwindow();
 
-	g_return_val_if_fail(dialog != NULL, -1);
+	cm_return_val_if_fail(dialog != NULL, -1);
 
 	send_send_data_progressive(session, len, len, dialog);
+	if (mainwin) {
+		gtk_widget_hide(mainwin->progressbar);
+		gtk_progress_bar_set_fraction
+			(GTK_PROGRESS_BAR(mainwin->progressbar),(gfloat)0);
+	}
+
 	return 0;
 }
 
 static void send_progress_dialog_size_allocate_cb(GtkWidget *widget,
 					 GtkAllocation *allocation)
 {
-	g_return_if_fail(allocation != NULL);
+	cm_return_if_fail(allocation != NULL);
 
 	prefs_common.sendwin_width = allocation->width;
 	prefs_common.sendwin_height = allocation->height;
@@ -574,7 +577,7 @@ static SendProgressDialog *send_progress_dialog_create(void)
 	gtk_widget_set_size_request(progress->window, prefs_common.sendwin_width,
 				    prefs_common.sendwin_height);
 
-	if (prefs_common.send_dialog_mode == SEND_DIALOG_ALWAYS) {
+	if (!prefs_common.send_dialog_invisible) {
 		gtk_widget_show_now(progress->window);
 	}
 	
@@ -585,8 +588,8 @@ static SendProgressDialog *send_progress_dialog_create(void)
 
 static void send_progress_dialog_destroy(SendProgressDialog *dialog)
 {
-	g_return_if_fail(dialog != NULL);
-	if (prefs_common.send_dialog_mode == SEND_DIALOG_ALWAYS) {
+	cm_return_if_fail(dialog != NULL);
+	if (!prefs_common.send_dialog_invisible) {
 		progress_dialog_destroy(dialog->dialog);
 	}
 	g_free(dialog);
@@ -595,6 +598,7 @@ static void send_progress_dialog_destroy(SendProgressDialog *dialog)
 static void send_cancel_button_cb(GtkWidget *widget, gpointer data)
 {
 	SendProgressDialog *dialog = (SendProgressDialog *)data;
+	statusbar_progress_all(0,0,0);
 
 	dialog->cancelled = TRUE;
 }
@@ -638,7 +642,9 @@ static void send_put_error(Session *session)
 			err_msg = g_strdup(log_msg);
 			break;
 		case SESSION_TIMEOUT:
-			log_msg = _("Session timed out.");
+			log_msg = _("Session timed out. You may be able to "
+				    "recover by increasing the timeout value in "
+				    "Preferences/Other/Miscellaneous.");
 			err_msg = g_strdup(log_msg);
 			break;
 		default:
